use Grid::GPT::Setup;
use Globus::Core::Paths;
use Getopt::Long;

# default values of config parameters
my $mjfs_service_name   = 'base/gram/ForkManagedJobFactoryService';
my $mjfs_name           = 'Fork ManagedJob Factory Service';
my $gt3_location        = $ENV{GLOBUS_LOCATION};
my $mjfs_manager_type   = 'fork';
my $mjfs_monitor_type   = 'poll';
my $package_name = '';
my ($globus_host_cputype, $globus_host_manufacturer, $globus_host_osname,
    $globus_host_osversion) = &get_system_info();

GetOptions('service-name|s=s' => \$mjfs_service_name,
           'name|n=s' => \$mjfs_name,
           'manager-type|t=s' => \$mjfs_manager_type,
           'monitor-type|r=s' => \$mjfs_monitor_type,
           'globus-location|g=s' => \$gt3_location,
           'cputype|c=s' => \$globus_host_cputype,
           'manufacturer|m=s' => \$globus_host_manufacturer,
           'os-name|o=s' => \$globus_host_osname,
           'os-version|v=s' => \$globus_host_osversion,
           'package-name|p=s' => \$package_name,
           'condor-arch|a=s' => \$condor_arch,
           'condor-os|O=s' => \$condor_os,
	   'help|h' => \$help);

&usage if ($help || $package_name eq '');

my $metadata =
    new Grid::GPT::Setup(package_name => $package_name);

my $globusdir	= $ENV{GLOBUS_LOCATION};

my @cmd = ('ant', '-f', "$ENV{GLOBUS_LOCATION}/etc/build-mjs-gar.xml",
   "-Dmjfs.service.name=$mjfs_service_name",
   "-Dmjfs.name=$mjfs_name",
   "-Dogsa.root=$gt3_location",
   "-Dmjfs.manager.type=$mjfs_manager_type",
   "-Dmjfs.monitor.type=$mjfs_monitor_type",
   "-Dmjfs.host.cputype=$globus_host_cputype",
   "-Dmjfs.host.manufacturer=$globus_host_manufacturer",
   "-Dmjfs.host.osname=$globus_host_osname",
   "-Dmjfs.host.osversion=$globus_host_osversion",
   "-Dabsolute.ogsa.root=$ENV{GLOBUS_LOCATION}",
   "-Dmjfs.condor.arch=$condor_arch",
   "-Dmjfs.condor.os=$condor_os",
   "deployGar");

$rc = system(@cmd);
if($? == 0)
{
    $metadata->finish();
}
else
{
    print STDERR "Error creating service entry $mjfs_name.\n";
}

sub usage
{
    print "Usage: $0 [options]\n".
	  "          [--help|-h]\n";
    exit 1;
}

sub get_system_info
{
    my ($cpu, $manufacturer, $os_name, $os_version);

    print "Determining system information...\n";
    ($cpu, $manufacturer) = (split(/-/, `${sbindir}/config.guess`))[0,1];
    $uname_cmd = &lookup_shell_command("GLOBUS_SH_UNAME");

    chomp($os_name=`$uname_cmd -s`);
    $os_version="";

    if($os_name eq "AIX")
    {
        chomp($os_version = `$uname_cmd -v`);
        $os_version .= ".";
    }

    chomp($os_version .= `$uname_cmd -r`);

    return ($cpu, $manufacturer, $os_name, $os_version);
}

sub lookup_shell_command
{
    my ($cmdvar, $cmd);
    $cmdvar = $_[0];

    chomp($cmd = `$bindir/globus-sh-exec -e echo \\\$$cmdvar`);

    return $cmd;
}

