/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.guide.impl;

import org.globus.ogsa.guide.Counter.CounterPortType;
import org.globus.ogsa.guide.Counter.CounterServiceLocator;
import org.globus.ogsa.guide.Counter.state.CounterStateType;
import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.MessageUtils;
import org.globus.ogsa.utils.QueryHelper;

import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.GridService;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.OGSIServiceGridLocator;
import org.gridforum.ogsi.ServiceDataValuesType;

import java.net.URL;


public class CounterClient {
    private static String addString =
        MessageUtils.getMessage(GuideConstants.GUIDE, "add");
    private static String subtractString =
        MessageUtils.getMessage(GuideConstants.GUIDE, "subtract");
    private static String valueString =
        MessageUtils.getMessage(GuideConstants.GUIDE, "value");
    private static String stateString =
        MessageUtils.getMessage(GuideConstants.GUIDE, "state");
    private static String statusString =
        MessageUtils.getMessage(GuideConstants.GUIDE, "status");
    private static String timestampString =
        MessageUtils.getMessage(GuideConstants.GUIDE, "timestamp");

    public static void main(String[] args) {
        if (args.length < 2) {
            System.err.println(
                MessageUtils.getMessage(
                    GuideConstants.GUIDE, "usage",
                    new String[] {
                        "CounterClient <handle> |add|subtract|get|state| |val|"
                    }
                )
            );

            return;
        }

        int val = 1;

        if (args.length > 2) {
            val = Integer.parseInt(args[2]);
        }

        try {
            CounterServiceLocator counterLocator = new CounterServiceLocator();
            OGSIServiceGridLocator gridLocator = new OGSIServiceGridLocator();
            GridService gridService =
                gridLocator.getGridServicePort(new HandleType(args[0]));
            CounterPortType counter =
                counterLocator.getCounterService(gridLocator.getEndpoint());

            if (args[1].equalsIgnoreCase("add")) {
                System.out.println("Counter " + addString + counter.add(val));
            } else if (args[1].equalsIgnoreCase("subtract")) {
                System.out.println(
                    "Counter " + subtractString + counter.subtract(val)
                );
            } else if (args[1].equalsIgnoreCase("get")) {
                System.out.println(
                    "Counter " + valueString + counter.getValue()
                );
            } else if (args[1].equalsIgnoreCase("state")) {
                ExtensibilityType extensibility =
                    gridService.findServiceData(
                        QueryHelper.getNamesQuery("CounterState")
                    );
                ServiceDataValuesType serviceData =
                    AnyHelper.getAsServiceDataValues(extensibility);
                CounterStateType counterState =
                    (CounterStateType) AnyHelper.getAsSingleObject(
                        serviceData, CounterStateType.class
                    );
                System.out.println("Counter " + stateString);
                System.out.println(
                    "    " + statusString + counterState.getStatus()
                );
                System.out.println(
                    "    " + valueString + counterState.getValue()
                );
                System.out.println(
                    "    " + timestampString +
                    counterState.getTimestamp().getTime()
                );
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
