/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.guide.impl;

import org.globus.ogsa.impl.ogsi.GridServiceImpl;
import org.globus.ogsa.guide.TimedCounter.TimedCounterPortType;

import java.rmi.RemoteException;

import java.util.Calendar;


public class ServiceDataAnnotationCounterImpl extends GridServiceImpl
    implements TimedCounterPortType {
    private int val = 0;
    private Calendar timeLastChange = Calendar.getInstance();

    public ServiceDataAnnotationCounterImpl() {
        super("Guide Service Data Counter");
    }

    public int add(int x) throws RemoteException {
        val += x;
        timeLastChange = Calendar.getInstance();

        return val;
    }

    public int subtract(int x) throws RemoteException {
        val -= x;
        timeLastChange = Calendar.getInstance();

        return val;
    }

    /**
     * The current value of the counter.
     * @ogsa:service-data
     *     name = "currentValue"
     *     minOccurs = "1"
     *     maxOccurs = "1"
     *     mutability = "mutable"
     *     modifiable = "true"
     *     nillable = "false"
     */
    public int getValue() throws RemoteException {
        return val;
    }

    /**
     * The current value of the counter and the time when it was most recently changed.
     * @ogsa:service-data
     */
    public CounterTimestamp getTimestamp() throws RemoteException {
        CounterTimestamp ts = new CounterTimestamp();
        ts.setValue(val);
        ts.setTimestamp(timeLastChange);

        return ts;
    }
}
