/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.guide.impl;

import org.globus.ogsa.impl.ogsi.GridServiceImpl;
import org.globus.ogsa.GridContext;
import org.globus.ogsa.GridServiceException;
import org.globus.ogsa.ServiceData;
import org.globus.ogsa.guide.Counter.CounterPortType;
import org.globus.ogsa.guide.Counter.state.CounterStateType;
import org.globus.ogsa.utils.MessageUtils;

import javax.xml.namespace.QName;

import java.rmi.RemoteException;

import java.util.Calendar;


public class ServiceDataCounterImpl extends GridServiceImpl
    implements CounterPortType {
    private static final String COUNTER_STATE_NS =
        "http://www.globus.org/namespaces/2003/04/guide/counter";
    private int val = 0;
    private ServiceData statusData;
    private ServiceData stateData;
    private CounterStateType state = new CounterStateType();
    private String initializedStatus =
        MessageUtils.getMessage(GuideConstants.GUIDE, "initializedStatus");
    private String addStatus =
        MessageUtils.getMessage(GuideConstants.GUIDE, "addStatus");
    private String subtractStatus =
        MessageUtils.getMessage(GuideConstants.GUIDE, "subtractStatus");
    private String getValueStatus =
        MessageUtils.getMessage(GuideConstants.GUIDE, "getValueStatus");

    public ServiceDataCounterImpl() {
        super("Guide Service Data Counter");
    }

    public void postCreate(GridContext context) throws GridServiceException {
        super.postCreate(context);
        statusData = this.serviceData.create("CounterStatus");
        stateData =
            this.serviceData.create(
                new QName(COUNTER_STATE_NS, "CounterState")
            );
        statusData.setValue(initializedStatus);
        updateState();
        stateData.setValue(this.state);
        serviceData.add(statusData);
        serviceData.add(stateData);
    }

    private void updateState() {
        state.setStatus((String) statusData.getValue());
        state.setTimestamp(Calendar.getInstance());
        state.setValue(this.val);
    }

    public int add(int val) throws RemoteException {
        statusData.setValue(addStatus);
        this.val = this.val + val;
        updateState();

        return this.val;
    }

    public int subtract(int val) throws RemoteException {
        statusData.setValue(subtractStatus);
        this.val = this.val - val;
        updateState();

        return this.val;
    }

    public int getValue() throws RemoteException {
        statusData.setValue(getValueStatus);
        updateState();

        return val;
    }
}
