/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.guide.impl;

import org.globus.ogsa.impl.core.service.ServicePropertiesImpl;
import org.globus.ogsa.NotificationSinkCallback;
import org.globus.ogsa.client.managers.NotificationSinkManager;
import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.MessageUtils;

import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.ServiceDataValuesType;

import java.rmi.RemoteException;


public class StatusListener extends ServicePropertiesImpl implements Runnable,
    NotificationSinkCallback {
    private NotificationSinkManager manager;
    private String sink;

    public StatusListener(
        NotificationSinkManager manager,
        String source
    ) throws Exception {
        this.manager = manager;
        this.sink =
            manager.addListener(
                "CounterStatus", null, new HandleType(source), this
            );
    }

    public void deliverNotification(ExtensibilityType any)
        throws RemoteException {
        try {
            ServiceDataValuesType serviceData =
                AnyHelper.getAsServiceDataValues(any);
            String counterStatus =
                (String) AnyHelper.getAsSingleObject(serviceData);
            System.out.println(
                "Counter " +
                MessageUtils.getMessage(GuideConstants.GUIDE, "status") +
                counterStatus
            );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void run() {
        try {
            this.manager.removeListener(this.sink);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println(
                MessageUtils.getMessage(
                    GuideConstants.GUIDE, "usage",
                    new String[] { "StatusListener <source>" }
                )
            );

            return;
        }

        try {
            NotificationSinkManager manager =
                NotificationSinkManager.getManager();
            manager.startListening(NotificationSinkManager.MAIN_THREAD);

            StatusListener listener = new StatusListener(manager, args[0]);
            Runtime.getRuntime().addShutdownHook(new Thread(listener));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
