/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.guide.impl;

import org.globus.ogsa.guide.Counter.complex.ActionType;
import org.globus.ogsa.guide.Counter.complex.ComplexCounterPortType;
import org.globus.ogsa.guide.Counter.complex.TimestampedValue;
import org.globus.ogsa.utils.MessageUtils;
import org.globus.ogsa.utils.WSIFUtils;

import org.apache.wsif.WSIFService;
import org.apache.wsif.WSIFServiceFactory;

import java.util.Calendar;


public class WSIFCounterClient {
    private static final String GUIDE_NS =
        "http://www.globus.org/namespaces/2003/04/guide/counter/complex";

    public static void main(String[] args) {
        if (args.length < 2) {
            System.err.println(
                MessageUtils.getMessage(
                    GuideConstants.GUIDE, "usage",
                    new String[] { "WSIFCounterClient value <wsdl url>" }
                )
            );

            return;
        }

        try {
            WSIFServiceFactory factory = WSIFServiceFactory.newInstance();
            WSIFService service =
                factory.getService(
                    args[1], null, null, GUIDE_NS, "ComplexCounterPortType"
                );
            WSIFUtils.registerMappings(service, ComplexCounterPortType.class);

            ComplexCounterPortType counter =
                (ComplexCounterPortType) service.getStub(
                    ComplexCounterPortType.class
                );
            TimestampedValue timestampedValue = new TimestampedValue();
            timestampedValue.setTime(Calendar.getInstance());
            timestampedValue.setValue(Integer.parseInt(args[0]));

            int val = counter.submitAction(ActionType.add, timestampedValue);
            System.out.println(
                MessageUtils.getMessage(GuideConstants.GUIDE, "value") + val
            );
        } catch (Exception e) {
            System.err.println(MessageUtils.toString(e));
        }
    }
}
