#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file globus_ogsa_security_authentication.h
 * Globus OGSA Security Authentication Library
 * @author Sam Lang
 *
 * $RCSfile: globus_ogsa_security_authentication.h,v $
 * $Revision: 1.4 $
 * $Date: 2003/12/08 21:32:58 $
 */
#endif

#ifndef _GLOBUS_OGSA_SECURITY_AUTHENTICATION_H_
#define _GLOBUS_OGSA_SECURITY_AUTHENTICATION_H_

#include <globus_common.h>
#include <stdsoap2.h>
#include <globus_gss_assist.h>

EXTERN_C_BEGIN

#define GLOBUS_OGSA_SECURITY_AUTHENTICATION_MODULE \
        (&globus_i_ogsa_security_authentication_module)

extern
globus_module_descriptor_t       globus_i_ogsa_security_authentication_module;

#define SOAP_AUTH_MECH_GSSAPI_GSI_2002_04  "http://www.globus.org/2002/04/xmlenc#gssapi-gsi"

#define GLOBUS_OGSA_SECURITY_NULL_OPTIONS  0x0
#define GLOBUS_OGSA_SECURITY_ENABLE_GRIM   0x2
#define GLOBUS_OGSA_SECURITY_DELEGATE      0x4
#define GLOBUS_OGSA_SECURITY_IGNORE_GRIM   0x8

typedef struct globus_ogsa_security_authentication_handle_s *
               globus_ogsa_security_authentication_handle_t;

typedef struct globus_ogsa_security_auth_attr_s *
               globus_ogsa_security_auth_attr_t;

globus_result_t
globus_ogsa_security_authentication_init_context(
    struct soap *                       soap,
    const char *                        url,
    const gss_cred_id_t                 cred_handle,
    gss_ctx_id_t *                      context_handle,
    char **                             context_id,
    char *                              target_name_string,
    OM_uint32                           req_flags,
    OM_uint32 *                         ret_flags);

globus_result_t
globus_ogsa_security_authentication_init(
    globus_ogsa_security_auth_attr_t security_attributes,
    char *                              url,
    gss_ctx_id_t *                      context,
    char **                             context_id);


globus_result_t
globus_ogsa_security_authentication_attr_init(
    globus_ogsa_security_auth_attr_t *  security_attributes,
    gss_cred_id_t                       cred_handle,
    char *                              target,
    int                                 options,
    char **                             port_types,
    char *                              username);

globus_result_t
globus_ogsa_security_authentication_attr_destroy(
    globus_ogsa_security_auth_attr_t    security_attributes);

globus_result_t
globus_ogsa_security_authentication_attr_copy(
    globus_ogsa_security_auth_attr_t    src,
    globus_ogsa_security_auth_attr_t *  dest);

globus_result_t
globus_ogsa_security_authentication_attr_set_port_types(
    globus_ogsa_security_auth_attr_t    attr,
    char **                             port_types);

globus_result_t
globus_ogsa_security_authentication_attr_set_username(
    globus_ogsa_security_auth_attr_t    attr,
    char *                              username);

globus_result_t
globus_ogsa_security_authentication_attr_set_target(
    globus_ogsa_security_auth_attr_t    attr,
    char *                              target);

globus_result_t
globus_ogsa_security_authentication_attr_set_options(
    globus_ogsa_security_auth_attr_t    attr,
    int                                 options);


globus_result_t
globus_ogsa_security_authentication_handle_init(
    globus_ogsa_security_authentication_handle_t * handle);

globus_result_t
globus_ogsa_security_authentication_handle_destroy(
    globus_ogsa_security_authentication_handle_t handle);

globus_result_t
globus_ogsa_security_authentication_get_context(
    globus_ogsa_security_authentication_handle_t handle,
    gss_ctx_id_t *                               gss_context);

globus_result_t
globus_ogsa_security_authentication_serve_initTokenExchange(
    globus_ogsa_security_authentication_handle_t handle,
    globus_ogsa_security_auth_attr_t             attrs,
    struct soap *                                soap);

globus_result_t
globus_ogsa_security_authentication_deserialize_continueTokenExchange(
    struct soap *                       soap,
    char **                             context_id,
    gss_buffer_t                        input_buffer);

globus_result_t
globus_ogsa_security_authentication_accept_continueTokenExchange(
    globus_ogsa_security_authentication_handle_t handle,
    globus_ogsa_security_auth_attr_t             attrs,
    struct soap *                                soap,
    gss_buffer_t                                 input_buffer);

globus_result_t
globus_ogsa_security_authentication_get_context_id(
    globus_ogsa_security_authentication_handle_t handle,
    char **                                      context_id);

int
globus_ogsa_security_authentication_continue_needed(
    globus_ogsa_security_authentication_handle_t handle);

EXTERN_C_END

#endif /* _OGSA_IMPL_SECURITY_AUTHENTICATION_H_ */
