#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file globus_ogsa_security_xml_plugin.h
 * Globus OGSA Security WSSE GSSAPI library
 * @author Sam Lang
 *
 * $RCSfile: globus_ogsa_security_wsse_gssapi.h,v $
 * $Revision: 1.1.1.1 $
 * $Date: 2003/06/04 20:09:14 $
 */
#endif

#ifndef _GLOBUS_OGSA_SECURITY_WSSE_GSSAPI_H_
#define _GLOBUS_OGSA_SECURITY_WSSE_GSSAPI_H_

#include <globus_common.h>
#include <stdsoap2.h>
#include <gssapi.h>
#include <globus_openssl.h>
#include <xmlsec/xmlenc.h>
#include <xmlsec/xmldsig.h>

EXTERN_C_BEGIN

#define GLOBUS_OGSA_SECURITY_WSSE_GSSAPI_MODULE \
        (&globus_l_ogsa_security_wsse_gssapi_module)

extern
globus_module_descriptor_t          globus_l_ogsa_security_wsse_gssapi_module;

#define GLOBUS_OGSA_SECURITY_WSSE_GSSAPI_PLUGIN_ID \
                                        "globus_ogsa_security_wsse_gssapi"

typedef struct globus_l_ogsa_security_wsse_gssapi_handle_s *
globus_ogsa_security_wsse_gssapi_handle_t;

globus_result_t
globus_ogsa_security_wsse_gssapi_get_gss_context(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    const char *                              context_id,
    gss_ctx_id_t *                            context);

globus_result_t
globus_ogsa_security_wsse_gssapi_add_gss_context(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    char *                                    context_id,
    gss_ctx_id_t                              context);

globus_result_t
globus_ogsa_security_wsse_gssapi_remove_gss_context(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    char *                                    context_id,
    gss_ctx_id_t *                            gss_context);

globus_result_t
globus_ogsa_security_wsse_gssapi_handle_init(
    globus_ogsa_security_wsse_gssapi_handle_t * handle);

globus_result_t
globus_ogsa_security_wsse_gssapi_handle_destroy(
    globus_ogsa_security_wsse_gssapi_handle_t handle);

globus_result_t
globus_ogsa_security_wsse_gssapi_set_gss_context(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    gss_ctx_id_t                              gss_context);

globus_result_t
globus_ogsa_security_wsse_gssapi_set_confidentiality(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    int                                       conf);

globus_result_t
globus_ogsa_security_wsse_gssapi_get_confidentiality(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    int *                                     conf);

globus_result_t
globus_ogsa_security_wsse_gssapi_wrap(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    char *                                    context_id,
    char *                                    orig,
    size_t                                    orig_length,
    char  *                                   buffer,
    size_t                                    buffer_length,
    char **                                   result_buff,
    size_t *                                  result_buff_length);

globus_result_t
globus_ogsa_security_wsse_gssapi_unwrap(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    char **                                   buffer,
    size_t *                                  buffer_length);

globus_result_t
globus_ogsa_security_wsse_gssapi_sign(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    char *                                    context_id,
    char *                                    orig,
    size_t                                    orig_length,
    char  *                                   buffer,
    size_t                                    buffer_length,
    char **                                   result_buff,
    size_t *                                  result_buff_length);

globus_result_t
globus_ogsa_security_wsse_gssapi_sign_dom(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    xmlDocPtr                                 doc,
    const char *                              context_id);

globus_result_t
globus_ogsa_security_wsse_gssapi_verify(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    char **                                   buff,
    size_t *                                  buff_length);

globus_result_t
globus_ogsa_security_wsse_gssapi_verify_dom(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    xmlDocPtr                                 doc);

globus_result_t
globus_ogsa_security_wsse_gssapi_encrypt(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    char *                                    context_id,
    char *                                    orig,
    size_t                                    orig_length,
    char  *                                   buff,
    size_t                                    buff_length,
    char **                                   result_buff,
    size_t *                                  result_buff_length);

globus_result_t
globus_ogsa_security_wsse_gssapi_encrypt_dom(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    xmlDocPtr                                 doc,
    char *                                    context_id);

globus_result_t
globus_ogsa_security_wsse_gssapi_decrypt(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    char **                                   buff,
    size_t *                                  buff_length);

globus_result_t
globus_ogsa_security_wsse_gssapi_decrypt_dom(
    globus_ogsa_security_wsse_gssapi_handle_t handle,
    xmlDocPtr                           doc);

int
globus_ogsa_security_wsse_gssapi_register(
    struct soap *                       soap,
    struct soap_plugin *                p,
    void *                              arg);

EXTERN_C_END

#endif /* _OGSA_IMPL_SECURITY_WSSE_GSSAPI_H_ */
