#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file globus_ogsa_utils.h
 * OGSA Impl Utility Library
 * @author Sam Lang
 *
 * $RCSfile: globus_ogsa_utils.h,v $
 * $Revision: 1.3 $
 * $Date: 2003/11/05 05:17:12 $
 */
#endif

#ifndef _GLOBUS_OGSA_UTILS_H_
#define _GLOBUS_OGSA_UTILS_H_

#define XSD_NS_HREF                     "http://www.w3.org/2001/XMLSchema"
#define XSI_NS_HREF                     "http://www.w3.org/2001/XMLSchema-instance"
#define SOAPENV_NS_HREF                 "http://schemas.xmlsoap.org/soap/envelope/"
#define SOAPENC_NS_HREF                 "http://schemas.xmlsoap.org/soap/encoding/"

#include <stdarg.h>
#include <string.h>
#include <stdsoap2.h>
#include <globus_io.h>
#include <libxml/tree.h>

EXTERN_C_BEGIN

#define GLOBUS_OGSA_UTILS_MODULE  (&globus_i_ogsa_utils_module)

extern
globus_module_descriptor_t              globus_i_ogsa_utils_module;

globus_result_t
globus_ogsa_utils_gsoap_handle_add_namespace(
    struct soap *                       soap,
    struct Namespace *                  namespaces);

globus_result_t
globus_ogsa_utils_get_time_from_string(
    const char *                        time_string,
    globus_abstime_t *                  abstime);

globus_result_t
globus_ogsa_utils_create_string_from_time(
    globus_abstime_t                    abstime,
    char **                             time_string);                       

globus_result_t
globus_ogsa_utils_soap_fault_check(
    struct soap *                       soap);

globus_result_t
globus_ogsa_utils_dump_soap_message(
    const char *                        orig_buffer,
    size_t                              length,
    xmlDocPtr                           doc,
    unsigned char **                    result_buffer,
    size_t *                            result_length);

globus_result_t
globus_ogsa_utils_parse_soap_message(
    const char *                        buffer,
    size_t                              length,
    xmlDocPtr *                         xmlDoc);

globus_result_t
globus_ogsa_utils_soap_fault(
    xmlDocPtr                           doc);

globus_result_t
globus_ogsa_utils_split_url(
    char *                              url,
    char **                             hostname,
    int *                               port,
    char **                             path);

globus_result_t
globus_ogsa_utils_check_httpg(
    char *                              url);

globus_result_t
globus_ogsa_utils_base64_encode(
    unsigned char *                     input_buffer,
    int                                 input_length,
    unsigned char **                    output_buffer);

globus_result_t
globus_ogsa_utils_base64_decode(
    unsigned char *                     input_buffer,
    unsigned char **                    output_buffer,
    int *                               output_length);

globus_result_t
globus_ogsa_utils_generate_uuid(
    char **                             uuid);

globus_result_t
globus_ogsa_utils_gsoap_remove_plugin(
    struct soap *                       soap,
    char *                              id);

globus_result_t
globus_ogsa_utils_gsoap_push_namespaces(
    struct Namespace **                 namespace_list,
    int *                               namespace_list_length,
    struct Namespace *                  namespaces);

globus_result_t
globus_ogsa_utils_gsoap_handle_init(
    struct soap **                      soap,
    const char *                        default_namespace,
    int                                 namespace_list_count,
    ...);

globus_result_t
globus_ogsa_utils_gsoap_handle_destroy(
    struct soap *                       soap);

globus_result_t
globus_ogsa_utils_libxml_find_node(
    xmlNodePtr                          node,
    const char *                        name,
    const char *                        ns,
    xmlNodePtr *                        found);

globus_result_t
globus_ogsa_utils_libxml_check_node_name(
    xmlNodePtr                          node,
    const char *                        name,
    const char *                        ns);

char *
globus_ogsa_utils_time_convert(
    globus_abstime_t *                  abstime);

EXTERN_C_END

#endif
