
#ifndef _GLOBUS_OGSA_UTILS_ERROR_H_
#define _GLOBUS_OGSA_UTILS_ERROR_H_

#include <globus_error.h>
#include <globus_error_openssl.h>

/* ERROR MACROS */

#define MAX_SOAP_ERROR 26
extern const char * globus_ogsa_utils_gsoap_error_strings[];

#define GLOBUS_OGSA_UTILS_GSOAP_ERROR(_RESULT_, _GSOAP_ERR_, _ERR_CODE_, \
                                      _ERRSTR_, _MODULE_, \
                                      _DEBUG_ERRSTR_) \
    { \
        globus_object_t *               _error_obj_; \
        char *                          _tmp_str_ = \
            globus_common_create_string _ERRSTR_; \
        if(_GSOAP_ERR_ >= 0 && _GSOAP_ERR_ <= MAX_SOAP_ERROR) \
        { \
            _error_obj_ = globus_error_construct_error( \
                _MODULE_, NULL, _GSOAP_ERR_, \
                __FILE__, _function_name_, __LINE__, \
                "GSOAP ERROR: %s", \
                globus_ogsa_utils_gsoap_error_strings[_GSOAP_ERR_]); \
        } \
        else \
        { \
            _error_obj_ = globus_error_construct_error( \
                _MODULE_, NULL, _GSOAP_ERR_, \
                __FILE__, _function_name_, __LINE__, \
                "GSOAP ERROR: Unknown GSOAP ERROR: %d", \
                _GSOAP_ERR_); \
        } \
        _error_obj_ = globus_error_construct_error( \
            _MODULE_, _error_obj_, _ERR_CODE_, \
            __FILE__, _function_name_, __LINE__, \
            "%s", \
            _tmp_str_); \
        free(_tmp_str_); \
        if(_DEBUG_ERRSTR_) \
        { \
            globus_error_set_long_desc(_error_obj_, _DEBUG_ERRSTR_); \
        } \
        _RESULT_ = globus_error_put(_error_obj_); \
    }

#define GLOBUS_OGSA_UTILS_OPENSSL_ERROR(_RESULT_, _ERR_CODE_, \
                                        _ERRSTR_, _MODULE_, \
                                        _DEBUG_ERRSTR_) \
    { \
        globus_object_t *               _error_obj_; \
        char *                          _tmp_str_ = \
            globus_common_create_string _ERRSTR_; \
        _error_obj_ = globus_error_wrap_openssl_error( \
            _MODULE_, \
            _ERR_CODE_, \
            __FILE__, _function_name_, __LINE__, \
            "%s", \
            _tmp_str_); \
        free(_tmp_str_); \
        if(_DEBUG_ERRSTR_) \
        { \
            globus_error_set_long_desc(_error_obj_, _DEBUG_ERRSTR_); \
        } \
        _RESULT_ = globus_error_put(_error_obj_); \
    } 

#define GLOBUS_OGSA_UTILS_ERROR(_RESULT_, _ERRORTYPE_, _ERRSTR_, \
                                _MODULE_, _DEBUG_ERRSTR_) \
    { \
        globus_object_t *               _error_obj_; \
        char *                          _tmp_str_ = \
            globus_common_create_string _ERRSTR_; \
        _error_obj_ = globus_error_construct_error( \
            _MODULE_, NULL, \
            _ERRORTYPE_, \
            __FILE__, _function_name_, __LINE__, \
            "%s", \
            _tmp_str_); \
        free(_tmp_str_); \
        if(_DEBUG_ERRSTR_) \
        { \
            globus_error_set_long_desc(_error_obj_, _DEBUG_ERRSTR_); \
        } \
        _RESULT_ = globus_error_put(_error_obj_); \
    }

#define GLOBUS_OGSA_UTILS_CHAIN_ERROR(_RESULT_, _ERRORTYPE_, _ERRSTR_, \
                                      _MODULE_, _DEBUG_ERRSTR_) \
    { \
        globus_object_t *               _error_obj_; \
        char *                          _tmp_str_ = \
            globus_common_create_string _ERRSTR_; \
        _error_obj_ = globus_error_construct_error( \
            _MODULE_, globus_error_get(_RESULT_), \
            _ERRORTYPE_, \
            __FILE__, _function_name_, __LINE__, \
            "%s", \
            _tmp_str_); \
        free(_tmp_str_); \
        if(_DEBUG_ERRSTR_) \
        { \
            globus_error_set_long_desc(_error_obj_, _DEBUG_ERRSTR_); \
        } \
        _RESULT_ = globus_error_put(_error_obj_); \
    }

#endif /* _OGSA_IMPL_ERROR_H_ */
