#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file ogsa_impl_debug.h
 * Globus OGSA IMPL Utilities Library
 * @author Sam Lang
 *
 * $RCSfile: ogsa_impl_debug.h,v $
 * $Revision: 1.4 $
 * $Date: 2004/01/26 23:09:36 $
 */
#endif

#ifndef _OGSA_IMPL_DEBUG_H_
#define _OGSA_IMPL_DEBUG_H_

#include <globus_common.h>
#include "ogsa_impl_utils.h"
#include <openssl/bio.h>

#ifdef BUILD_DEBUG

#define OGSA_IMPL_UTILS_DEBUG(_LEVEL_) \
    (OGSA_DEBUG_LEVEL_VARIABLE >= (_LEVEL_))

#define OGSA_IMPL_UTILS_DEBUG_FPRINTF(_LEVEL_, _MESSAGE_) \
    { \
        if (OGSA_IMPL_UTILS_DEBUG(_LEVEL_)) \
        { \
            globus_libc_fprintf _MESSAGE_; \
        } \
    }

#define OGSA_IMPL_UTILS_DEBUG_NPRINTF(_LEVEL_, _MESSAGE_) \
    { \
        if (OGSA_IMPL_UTILS_DEBUG(_LEVEL_)) \
        { \
           char *                       _tmp_str_ = \
               ogsa_impl_utils_create_nstring _MESSAGE_; \
           globus_libc_fprintf(OGSA_DEBUG_FSTREAM_VARIABLE, \
                               _tmp_str_); \
           globus_libc_free(_tmp_str_); \
        } \
    }

#define OGSA_IMPL_UTILS_DEBUG_PRINTF(_LEVEL_, _MESSAGE_) \
    { \
        if (OGSA_IMPL_UTILS_DEBUG(_LEVEL_)) \
        { \
           char *                       _tmp_str_ = \
               ogsa_impl_utils_create_string _MESSAGE_; \
           globus_libc_fprintf(OGSA_DEBUG_FSTREAM_VARIABLE, \
                               _tmp_str_); \
           globus_libc_free(_tmp_str_); \
        } \
    }

#define OGSA_IMPL_UTILS_DEBUG_PRINT(_LEVEL_, _MESSAGE_) \
    { \
        if (OGSA_IMPL_UTILS_DEBUG(_LEVEL_)) \
        { \
           globus_libc_fprintf(OGSA_DEBUG_FSTREAM_VARIABLE, _MESSAGE_); \
        } \
    }

#define OGSA_IMPL_UTILS_DEBUG_DUMP(_LEVEL_, _BYTES_, _LENGTH_) \
    { \
        if(OGSA_IMPL_UTILS_DEBUG(_LEVEL_)) \
        { \
            BIO *                       dbio; \
            dbio = BIO_new_fp(OGSA_DEBUG_FSTREAM_VARIABLE, BIO_NOCLOSE); \
            BIO_dump(dbio, _BYTES_, _LENGTH_); \
            BIO_free(dbio); \
        } \
    }

#else

#define OGSA_IMPL_UTILS_DEBUG(_LEVEL_) 0
#define OGSA_IMPL_UTILS_DEBUG_FPRINTF(_LEVEL_, _MESSAGE_) {}
#define OGSA_IMPL_UTILS_DEBUG_NPRINTF(_LEVEL_, _MESSAGE_) {}
#define OGSA_IMPL_UTILS_DEBUG_PRINTF(_LEVEL_, _MESSAGE_) {}
#define OGSA_IMPL_UTILS_DEBUG_PRINT(_LEVEL_, _MESSAGE_) {}
#define OGSA_IMPL_UTILS_DEBUG_DUMP(_LEVEL_, _BYTES_, _LENGTH_) {}

#endif
         
#define OGSA_IMPL_UTILS_DEBUG_ENTER \
            OGSA_IMPL_UTILS_DEBUG_FPRINTF( \
                2, (OGSA_DEBUG_FSTREAM_VARIABLE, \
                    "ENTERING: %s\n", _function_name_))

#define OGSA_IMPL_UTILS_DEBUG_EXIT \
            OGSA_IMPL_UTILS_DEBUG_FPRINTF( \
                2, (OGSA_DEBUG_FSTREAM_VARIABLE, \
                    "EXITING: %s\n", _function_name_))

#endif /* _OGSA_IMPL_DEBUG_H_ */
