/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.impl.security.SecurityManager;
import org.globus.ogsa.impl.security.Constants;
import org.globus.ogsa.impl.security.authorization.Authorization;
import org.globus.ogsa.impl.security.authorization.NoAuthorization;
import org.globus.ogsa.NotificationSinkCallback;
import org.globus.ogsa.client.managers.NotificationSinkManager;
import org.globus.ogsa.samples.chat.Chat;
import org.globus.ogsa.samples.chat.MessageData;
import org.globus.ogsa.samples.chat.service.ChatServiceLocator;
import org.globus.ogsa.utils.AnyHelper;

import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.ServiceDataValuesType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import javax.xml.rpc.Stub;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.rmi.RemoteException;

import java.util.Hashtable;


public class ChatPortTypePanel extends AbstractPortTypePanel
    implements NotificationSinkCallback {
    static Log logger = LogFactory.getLog(ChatPortTypePanel.class.getName());
    private Hashtable properties = new Hashtable();
    private NotificationSinkManager manager;
    private String sink;
    private JTextArea outputTextArea;
    private JTextField aliasTextField;
    private JTextField inputTextField;
    private JTextField commandTextField;
    private Chat chatService;

    public ChatPortTypePanel() {
        super(" Chat ");
    }

    public void init() {
        ChatServiceLocator chatServiceLocator = new ChatServiceLocator();

        try {
            this.chatService =
                chatServiceLocator.getChatPort(this.defaultEndpoint);
        } catch (javax.xml.rpc.ServiceException se) {
            logger.error("problem finding Chat service", se);
        }

        setLayout(new BorderLayout());

        this.outputTextArea = new JTextArea();

        JScrollPane scrollPane = new JScrollPane(this.outputTextArea);
        scrollPane.setPreferredSize(new Dimension(200, 150));
        add(scrollPane, BorderLayout.CENTER);

        JPanel inputPanel = new JPanel();
        inputPanel.setLayout(new BorderLayout());

        //Alias input
        JPanel aliasPanel = new JPanel();

        aliasPanel.add(new JLabel("Alias:"));

        this.aliasTextField = new JTextField(10);
        aliasPanel.add(aliasTextField);

        final ChatPortTypePanel chatPanel = this;

        JButton setAliasButton = new JButton("Set/Change Alias");
        setAliasButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    chatPanel.context.setAuthentication(
                        (Stub) chatPanel.chatService
                    );

                    String requestedAlias = chatPanel.aliasTextField.getText();

                    try {
                        chatPanel.chatService.setAlias(requestedAlias);
                    } catch (java.rmi.RemoteException re) {
                        JOptionPane.showMessageDialog(
                            null,
                            "There was a problem registering the " + "alias " +
                            requestedAlias + ":\n" + re.getMessage(),
                            "Remote Exception", JOptionPane.ERROR_MESSAGE
                        );
                        logger.error("problem registering alias", re);
                    }

                    chatPanel.inputTextField.setText("");
                }
            }
        );
        aliasPanel.add(setAliasButton);

        inputPanel.add(aliasPanel, BorderLayout.NORTH);

        //Message input
        JPanel messagePanel = new JPanel();

        messagePanel.add(new JLabel("Message:"));

        this.inputTextField = new JTextField(50);
        messagePanel.add(this.inputTextField);

        JButton sendButton = new JButton("Send Message");
        sendButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    chatPanel.context.setAuthentication(
                        (Stub) chatPanel.chatService
                    );

                    try {
                        chatPanel.chatService.sendMessage(
                            chatPanel.inputTextField.getText()
                        );
                    } catch (java.rmi.RemoteException re) {
                        JOptionPane.showMessageDialog(
                            null,
                            "There was a problem sending your message:\n" +
                            re.getMessage(), "Remote Exception",
                            JOptionPane.ERROR_MESSAGE
                        );
                        logger.error("problem sending chat message", re);
                    }

                    chatPanel.inputTextField.setText("");
                }
            }
        );
        messagePanel.add(sendButton);

        inputPanel.add(messagePanel, BorderLayout.CENTER);

        //Command input
        JPanel commandPanel = new JPanel();

        commandPanel.add(new JLabel("Command:"));

        this.commandTextField = new JTextField(50);
        commandPanel.add(commandTextField);

        JButton executeButton = new JButton("Execute Command");
        executeButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    chatPanel.context.setAuthentication(
                        (Stub) chatPanel.chatService
                    );

                    String commandResult = null;

                    try {
                        commandResult =
                            chatPanel.chatService.executeCommand(
                                chatPanel.commandTextField.getText()
                            );

                        chatPanel.outputTextArea.append(
                            "Executing command \"" +
                            chatPanel.commandTextField.getText() + "\":\n\t" +
                            commandResult + "\n"
                        );

                        chatPanel.commandTextField.setText("");
                    } catch (java.rmi.RemoteException re) {
                        JOptionPane.showMessageDialog(
                            null,
                            "There was a problem executing your " +
                            "command:\n" + re.getMessage(), "Remote Exception",
                            JOptionPane.ERROR_MESSAGE
                        );
                        logger.error("problem sending chat command", re);
                    }
                }
            }
        );
        commandPanel.add(executeButton);

        inputPanel.add(commandPanel, BorderLayout.SOUTH);

        add(inputPanel, BorderLayout.SOUTH);

        addTopicListener();
    }

    public void dispose() {
        try {
            if (this.manager.isListening()) {
                this.manager.removeListener(this.sink);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void addTopicListener() {
        // flag to disable self authorization so the msgs
        // can be delivered to anyone
        setProperty(Authorization.AUTHORIZATION, "none");

        Hashtable props = new Hashtable();
        props.put(Constants.GSI_SEC_CONV, 
		  Constants.ENCRYPTION);
	props.put(Constants.AUTHORIZATION, 
		  NoAuthorization.getInstance());
	
        try {
            this.manager = NotificationSinkManager.getInstance("Secure");
            this.manager.init(props);
            this.manager.startListening(NotificationSinkManager.CHILD_THREAD);
            this.sink =
                this.manager.addListener(
                    "ChatUpdate", null, new HandleType(this.handle.toString()),
                    this
                );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void deliverNotification(ExtensibilityType messageDataWrapper)
        throws RemoteException {
        String caller = SecurityManager.getManager().getCaller();

        if (caller == null) {
            throw new RemoteException("Caller not set");
        }

        try {
            ServiceDataValuesType serviceDataValues =
                AnyHelper.getAsServiceDataValues(messageDataWrapper);
            MessageData messageData =
                (MessageData) AnyHelper.getAsSingleObject(
                    serviceDataValues, MessageData.class
                );
            String alias = messageData.getAlias();
            String message = messageData.getText();
            this.outputTextArea.append(alias + ": " + message + "\n");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Object getProperty(String name) {
        return properties.get(name);
    }

    public Object getPersistentProperty(String name) {
        return null;
    }

    public void setPersistentProperty(
        String name,
        Object property
    ) {
    }

    public void flush() {
    }

    public void setProperty(
        String name,
        Object property
    ) {
        properties.put(name, property);
    }
}
