/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.samples.counter.CounterPortType;
import org.globus.ogsa.samples.counter.service.CounterServiceGridLocator;
import org.globus.ogsa.impl.security.authorization.NoAuthorization;
import org.globus.ogsa.impl.security.authorization.SelfAuthorization;
import org.globus.ogsa.impl.security.authentication.Constants;
import org.globus.ogsa.utils.GSIUtils;
import org.globus.axis.gsi.GSIConstants;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import javax.xml.rpc.Stub;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.net.URL;

public class CounterPortTypePanel extends AbstractPortTypePanel {
    private CounterServiceGridLocator locator;
    private JTextField valTF;

    public CounterPortTypePanel() {
        super(" Counter Example ");
        this.locator = new CounterServiceGridLocator();

        setLayout(new BorderLayout());

        this.valTF = new JTextField(25);

        JButton addBT = new JButton("Add");
        JButton subtractBT = new JButton("Subtract");
        JPanel valPanel = new JPanel();
        valPanel.setBorder(BorderFactory.createEtchedBorder());
        valPanel.add(new JLabel("Value: "));
        valPanel.add(this.valTF);
        valPanel.add(addBT);
        valPanel.add(subtractBT);
        addBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    add();
                }
            }
        );
        subtractBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    subtract();
                }
            }
        );

        add(valPanel, BorderLayout.NORTH);
    }

    private void add() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            CounterPortType counter =
                locator.getCounterPort(this.defaultEndpoint);
            setSecurityProperties((Stub) counter, this.defaultEndpoint);

            int val = counter.add(Integer.parseInt(this.valTF.getText()));
            this.valTF.setText(new Integer(val).toString());
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to add to counter: " + e.getMessage(),
                "Counter: add error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private void subtract() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            CounterPortType counter =
                locator.getCounterPort(this.defaultEndpoint);
            setSecurityProperties((Stub) counter, this.defaultEndpoint);

            int val = counter.subtract(Integer.parseInt(this.valTF.getText()));
            this.valTF.setText(new Integer(val).toString());
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to subtract from counter: " + e.getMessage(),
                "Counter: subtract error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private void getValue() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            CounterPortType counter =
                locator.getCounterPort(this.defaultEndpoint);
            setSecurityProperties((Stub) counter, this.defaultEndpoint);

            int val = counter.getValue();
            this.valTF.setText(new Integer(val).toString());
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to get value from counter: " + e.getMessage(),
                "Counter: getValue error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    public void setSecurityProperties(Stub stub,
				      URL endpoint) {
	if (endpoint.getProtocol().equals("httpg")) {
            stub._setProperty(GSIConstants.GSI_MODE, 
                              GSIConstants.GSI_MODE_LIMITED_DELEG);
            // assume SELF authorization
            stub._setProperty(Constants.AUTHORIZATION, 
                              SelfAuthorization.getInstance());
        } else {
            this.context.setAuthentication(stub);
        }
    }

    public void init() {
	getValue();
    }
    
    public void dispose() {
    }
}
