/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.swing.JButton;
import javax.swing.JPanel;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.Vector;


public class DynamicArrayPanel extends JPanel {
    protected static Log logger =
        LogFactory.getLog(DynamicArrayPanel.class.getName());
    Class parameterType = null;
    Vector parameterPanels = null;

    public DynamicArrayPanel(Class parameterType) {
        this.parameterType = parameterType;
        this.parameterPanels = new Vector();

        JButton addElementButton = new JButton("Add Element");
        addElementButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    try {
                        addElement();
                    } catch (Exception e) {
                        logger.error("operation invocation failed", e);
                    }
                }
            }
        );
        add(addElementButton);

        JButton removeElementButton = new JButton("Remove Element");
        removeElementButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    try {
                        removeElement();
                    } catch (Exception e) {
                        logger.error("operation invocation failed", e);
                    }
                }
            }
        );
        add(removeElementButton);
    }

    private void addElement() {
        //add new DynamicParameterPanel
        DynamicParameterPanel parameterPanel =
            new DynamicParameterPanel(
                String.valueOf(parameterPanels.size()),
                this.parameterType.getComponentType()
            );
        add(parameterPanel);
        this.parameterPanels.add(parameterPanel);

        Container child = this;
        Container parent = this.getParent();

        while (parent != null) {
            child = parent;
            parent = child.getParent();
        }

        child.repaint();
    }

    private void removeElement() {
        //add new DynamicParameterPanel
        DynamicParameterPanel parameterPanel =
            (DynamicParameterPanel) this.parameterPanels.elementAt(
                this.parameterPanels.size() - 1
            );
        remove(parameterPanel);
        parameterPanels.remove(parameterPanel);

        Container child = this;
        Container parent = this.getParent();

        while (parent != null) {
            child = parent;
            parent = child.getParent();
        }

        child.repaint();
    }

    public Object[] getArray() {
        Object[] parameterValueArray = new Object[parameterPanels.size()];

        Iterator parameterPanelIterator = parameterPanels.iterator();

        for (int index = 0; parameterPanelIterator.hasNext(); index++) {
            DynamicParameterPanel parameterPanel =
                (DynamicParameterPanel) parameterPanelIterator.next();

            try {
                parameterValueArray[index] = parameterPanel.getValue();
            } catch (Exception e) {
                logger.warn("failed to create value object", e);
                parameterValueArray[index] = null;
            }
        }

        return parameterValueArray;
    }

    public void setArray(Object parameterValue) {
        Object[] parameterValueArray = (Object[]) parameterValue;

        int panelCount = this.parameterPanels.size();

        while (panelCount < parameterValueArray.length) {
            addElement();
            panelCount++;
        }

        for (int index = 0; index < parameterValueArray.length; index++) {
            DynamicParameterPanel parameterPanel =
                (DynamicParameterPanel) parameterPanels.elementAt(index);

            parameterPanel.setValue(parameterValueArray[index]);
        }
    }
}
