/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import com.ibm.wsdl.xml.WSDLWriterImpl;

import org.globus.ogsa.impl.core.handle.HandleHelper;
import org.globus.ogsa.GridServiceException;
import org.globus.ogsa.client.reflection.ReflectionException;
import org.globus.ogsa.client.reflection.ReflectionUtils;
import org.globus.ogsa.wsdl.GSR;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

import javax.wsdl.Definition;

import javax.xml.namespace.QName;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.StringWriter;

import java.net.URL;

import java.util.Iterator;
import java.util.Vector;


public class DynamicInvokerGUI extends JFrame {
    protected static Log logger =
        LogFactory.getLog(DynamicInvokerGUI.class.getName());
    JPanel mainPanel = null;
    JTextArea wsdlTextArea = null;
    JButton refreshButton = null;
    String serviceInstanceUrl = null;

    public DynamicInvokerGUI(
        String registryServiceUrlString,
        String serviceInstanceKey,
        String serviceFactoryUrlString
    ) {
        this.serviceInstanceUrl =
            serviceFactoryUrlString + "/" + serviceInstanceKey;

        //Kill the app when this frame closes
        final DynamicInvokerGUI parent = this;
        addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                    System.exit(0);
                }
            }
        );

        JPanel rootPanel = new JPanel();

        JTabbedPane tabbedPane = new JTabbedPane();

        this.mainPanel = new JPanel();
        this.mainPanel.setLayout(
            new BoxLayout(this.mainPanel, BoxLayout.Y_AXIS)
        );
        this.wsdlTextArea = new JTextArea();

        final DynamicInvokerGUI gui = this;
        this.refreshButton = new JButton("Refresh");
        this.refreshButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        GSR reference =
                            new GSR(gui.serviceInstanceUrl + "?wsdl");
                        String handle = reference.getHandle().toString();
                        gui.refresh(gui.serviceInstanceUrl);
                    } catch (Exception exception) {
                        logger.error("problem refreshing WSDL", exception);
                    }
                }
            }
        );

        try {
            refresh(this.serviceInstanceUrl);
        } catch (Exception exception) {
            logger.error("problem refreshing WSDL", exception);
        }

        tabbedPane.addTab("Service", new JScrollPane(this.mainPanel));
        tabbedPane.addTab("WSDL", new JScrollPane(this.wsdlTextArea));
        getContentPane().add(tabbedPane);
    }

    private void refresh(String serviceInstanceUrl) throws GridServiceException {
        //Erase the old interface
        Component[] components = this.mainPanel.getComponents();

        /*
        for ( int index=0; (index<components.length) && !(components[index] instanceof DynamicServicePanel); index++ ) {
            if ( components[index] instanceof DynamicServicePanel ) {
                this.mainPanel.remove( components[index] );
            }
        }
        */
        for (int index = 0; index < components.length; index++) {
            logger.debug("Removing component #" + index);
            this.mainPanel.remove(components[index]);
        }

        //Collect all operations for this service
        Vector operationTypes = new Vector();

        GSR gsr = null;

        try {
            gsr = new GSR(serviceInstanceUrl + "?wsdl");
        } catch (Exception e) {
            logger.error("failed to initialize GSR", e);
            System.exit(1);
        }

        Iterator portTypeIterator = gsr.getPortTypes().iterator();

        while (portTypeIterator.hasNext()) {
            QName portTypeName = (QName) portTypeIterator.next();

                try {
                    operationTypes.addAll(
                        ReflectionUtils.getOperationDescriptions(
                            gsr.getWSDLDefinition(),
                            gsr.getDocument(),
                            portTypeName
                        )
                    );
                } catch (ReflectionException re) {
                    logger.error("problem refreshing dynamic gui", re);
                    return;
                }
        }

        //Add a panel for this service
        this.mainPanel.add(
            new DynamicServicePanel(serviceInstanceUrl, operationTypes)
        );
        this.mainPanel.add(this.refreshButton);

        this.repaint();

        //Set the WSDL text
        Definition wsdlDefinition = gsr.getWSDLDefinition();
        WSDLWriterImpl wsdlWriter = new WSDLWriterImpl();
        StringWriter stringWriter = new StringWriter();

        try {
            wsdlWriter.writeWSDL(wsdlDefinition, stringWriter);
        } catch (javax.wsdl.WSDLException wsdle) {
            logger.error("problem generating wsdl for text area", wsdle);
        }

        this.wsdlTextArea.setText(stringWriter.toString());
    }

    public static void main(String[] args) {
        JFrame frame = new DynamicInvokerGUI(args[0], args[1], args[2]);

        frame.setSize(1000, 400);
        frame.setVisible(true);
    }
}
