/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.client.reflection.DynamicInvoker;
import org.globus.ogsa.client.reflection.ReflectionException;
import org.globus.ogsa.client.reflection.ReflectionUtils;

import org.apache.axis.description.OperationDesc;
import org.apache.axis.description.ParameterDesc;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.swing.JButton;
import javax.swing.JPanel;

import javax.xml.namespace.QName;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.Vector;


public class DynamicOperationPanel extends JPanel {
    protected static Log logger =
        LogFactory.getLog(DynamicOperationPanel.class.getName());
    private Vector operationListeners = new Vector();
    private String gsh = null;
    private OperationDesc operationDesc = null;
    protected Vector parameterPanels = null;
    protected DynamicParameterPanel returnValuePanel = null;

    public DynamicOperationPanel(
        String gsh,
        OperationDesc operationDesc
    ) {
        this.gsh = gsh;
        this.operationDesc = operationDesc;

        this.parameterPanels = new Vector();

        if (logger.isDebugEnabled()) {
            logger.debug("GSH: " + this.gsh);
        }

        int index = 0;
        ParameterDesc parameter = operationDesc.getParameter(index);

        while (parameter != null) {
            String parameterName = parameter.getName();

            QName parameterTypeQName = parameter.getTypeQName();
            Class parameterType = null;
            try {
                parameterType =
                    ReflectionUtils.getClassForQName(parameterTypeQName);
            } catch (ReflectionException re) {
                logger.error(
                    "failed to initialize dynamic operation panel",
                    re);
                return;
            }

            if (logger.isDebugEnabled()) {
                logger.debug("Parameter Type QName: " + parameterTypeQName);

                if (parameterType != null) {
                    logger.debug("Parameter Type: " + parameterType.toString());
                } else {
                    logger.error("unable to determine parameter type");
                }
            }

            DynamicParameterPanel parameterPanel =
                new DynamicParameterPanel(parameterName, parameterType);

            this.parameterPanels.add(parameterPanel);
            add(parameterPanel);

            parameter = operationDesc.getParameter(++index);
        }

        //Add return value panel
        //Class returnType = operationDesc.getJavaReturnType();
        Class returnType = null;
        try {
            returnType =
                ReflectionUtils.getClassForQName(operationDesc.getReturnType());
        } catch (ReflectionException re) {
            logger.error(
                "failed to initialize dynamic operation panel",
                re);
            return;
        }

        if (logger.isDebugEnabled()) {
            logger.debug("return type: " + returnType);
        }

        if (returnType != null) {
            this.returnValuePanel =
                new DynamicParameterPanel(
                    DynamicParameterPanel.RETURN_VALUE_NAME, returnType
                );
            add(returnValuePanel);
        }

        //Add invoke button
        JButton invokeButton = new JButton("Invoke " + operationDesc.getName());

        invokeButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    try {
                        invokeOperation();
                    } catch (Exception e) {
                        logger.error("operation invocation failed", e);
                    }
                }
            }
        );

        add(invokeButton);
    }

    private Object[] getParameterValues() {
        Object[] parameterValues = new Object[parameterPanels.size()];

        Iterator parameterPanelIterator = parameterPanels.iterator();

        for (int index = 0; parameterPanelIterator.hasNext(); index++) {
            DynamicParameterPanel parameterPanel =
                (DynamicParameterPanel) parameterPanelIterator.next();

            try {
                parameterValues[index] = parameterPanel.getValue();
            } catch (Exception e) {
                logger.warn("failed to create value object", e);
                parameterValues[index] = null;
            }
        }

        return parameterValues;
    }

    protected void invokeOperation() throws Exception {
        Object[] parameters = getParameterValues();

        Object returnValue =
            new DynamicInvoker(this.gsh).call(
                operationDesc.getName(), parameters
            );

        if (returnValue != null) {
            this.returnValuePanel.setValue(returnValue);
        } else if (this.returnValuePanel != null) {
            logger.error(
                "Expected return value from operation invocation," +
                " but got null"
            );
        }

        fireOperationFinished(operationDesc, returnValue);
    }

    public void addOperationListener(OperationListener listener) {
        operationListeners.add(listener);
    }

    public void removeOperationListener(OperationListener listener) {
        operationListeners.remove(listener);
    }

    public void fireOperationFinished(
        OperationDesc operationDesc,
        Object result
    ) {
        Iterator listenerIterator = operationListeners.iterator();

        while (listenerIterator.hasNext()) {
            OperationListener listener =
                (OperationListener) listenerIterator.next();

            listener.operationFinished(operationDesc, result);
        }
    }
}
