/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.client.reflection.ReflectionUtils;
import org.globus.ogsa.utils.MessageUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.swing.BoxLayout;
import javax.swing.JScrollPane;

import javax.xml.namespace.QName;

import java.util.Collection;
import java.util.Iterator;


public class DynamicPortTypePanel extends AbstractPortTypePanel {
    protected static Log logger =
        LogFactory.getLog(DynamicPortTypePanel.class.getName());

    public DynamicPortTypePanel() {
        super("Dynamic Port Type");

        //init();
    }

    public DynamicPortTypePanel(String title) {
        super(title);

        //init();
    }

    protected QName portTypeName = null;

    public void setPortType(QName portTypeName) {
        this.portTypeName = portTypeName;
    }

    public QName getPortType() {
        return this.portTypeName;
    }

    public void init() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        try {
            //If the port type was not explicitly set, we must pick a default
            if (this.portTypeName == null) {
                Collection portTypeNames = this.gsr.getPortTypes();
                Iterator portTypeNameIterator = portTypeNames.iterator();

                //Set the port type to the second one listed in the WSDL
                //if it exists, otherwise use the GridService port type
                if (portTypeNameIterator.hasNext()) {
                    QName gsPortTypeName = (QName) portTypeNameIterator.next();
                    if (logger.isDebugEnabled()) {
                        logger.debug("First Port Type: " + gsPortTypeName);
                    }

                    if (portTypeNameIterator.hasNext()) {
                        this.portTypeName = (QName) portTypeNameIterator.next();
                        if (logger.isDebugEnabled()) {
                            logger.debug("Second Port Type: "
                                        + this.portTypeName);
                        }
                    } else {
                        this.portTypeName = gsPortTypeName;
                    }
                }
            }
            logger.debug(" port type name: " + this.portTypeName);

            //Get all operations for this port type
            Collection operationTypes =
                ReflectionUtils.getOperationDescriptions(
                    this.gsr.getWSDLDefinition(), this.gsr.getDocument(),
                    this.portTypeName
                );

            //Add a panel for this port type
            if (operationTypes != null) {
                add(
                    new JScrollPane(
                        new DynamicServicePanel(
                            this.gsr.getHandle().toString(), operationTypes
                        )
                    )
                );
            }
        } catch (Exception e) {
            logger.error(
                "Failed to initialize dynamic gui: " +
                MessageUtils.toString(e)
            );
        }
    }
}
