/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.GSIUtils;
import org.globus.ogsa.utils.GridServiceFactory;
import org.globus.ogsa.utils.MessageUtils;
import org.globus.ogsa.wsdl.GSR;

import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.FaultType;
import org.gridforum.ogsi.LocatorType;
import org.gridforum.ogsi.OGSIServiceGridLocator;
import org.gridforum.ogsi.ServiceAlreadyExistsFaultType;

import javax.swing.JButton;
import javax.swing.JOptionPane;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


public class FactoryPortTypePanel extends AbstractPortTypePanel {

    private OGSIServiceGridLocator factoryService =
        new OGSIServiceGridLocator();

    public FactoryPortTypePanel() {
        super(" Factory ");

        JButton createBT = new JButton("Create Instance");

        createBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        createInstance();
                    } catch (Exception exc) {
                        exc.printStackTrace();
                    }
                }
            }
        );

        add(createBT);
    }

    private void createInstance() throws Exception {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            GridServiceFactory factory =
                new GridServiceFactory(
                    factoryService.getFactoryPort(this.defaultEndpoint)
                );

            GSIUtils.setDefaultGSIProperties(
                factory.getStub(), this.defaultEndpoint
            );

            this.context.setAuthentication(factory.getStub());

            LocatorType locator = factory.createService();

            GSR reference = GSR.newInstance(locator);
            String location = reference.getHandle().toString();

            ServiceBrowser browser = this.context.getBrowser();
            browser.newWindow(location);
        } catch (ServiceAlreadyExistsFaultType saef) {
            String[] descriptions = saef.getDescription();
            String description =
                ((descriptions != null) && (descriptions.length > 0))
                ? (": " + descriptions[0]) : "";
            ExtensibilityType extension = saef.getExtension();

            if (extension != null) {
                System.err.println(AnyHelper.getAsString(extension));
            }

            JOptionPane.showMessageDialog(
                this,
                "Failed to create instance: Service Already exists " +
                description, "Factory: createService error",
                JOptionPane.ERROR_MESSAGE
            );

            return;
        } catch (FaultType gsf) {
            String[] descriptions = gsf.getDescription();
            String description =
                ((descriptions != null) && (descriptions.length > 0))
                ? (": " + descriptions[0]) : "";
            ExtensibilityType extension = gsf.getExtension();

            if (extension != null) {
                System.err.println(AnyHelper.getAsString(extension));
            }

            JOptionPane.showMessageDialog(
                this,
                "Failed to create instance: Grid Service Fault: " +
                description, "Factory: createService error",
                JOptionPane.ERROR_MESSAGE
            );

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to create instance: " + e.getMessage(),
                "Factory: createService error", JOptionPane.ERROR_MESSAGE
            );
            System.err.println(MessageUtils.toString(e));

            return;
        }
    }
}
