/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.samples.google.GoogleSearchPort;
import org.globus.ogsa.samples.google.GoogleSearchResult;
import org.globus.ogsa.samples.google.GoogleSearchServiceLocator;
import org.globus.ogsa.samples.google.ResultElement;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.PrintWriter;
import java.io.StringWriter;


public class GoogleSearchPortTypePanel extends AbstractPortTypePanel {
    private JTextField spelling;
    private JTextField searchString;
    private JTextField cachedURL;
    private JLabel tempLabel;
    private JTextArea searchResults;
    private JTextArea cachedTextArea;
    private JScrollPane searchScrollPane;
    private String key = "wMFVucygk0f60Sw+/NjpM3TXw4lGb33P";

    public GoogleSearchPortTypePanel() {
        super("Google Search");
        this.spelling = new JTextField(10);
        this.searchString = new JTextField(30);
        this.cachedURL = new JTextField(20);
        this.searchResults = new JTextArea(25, 20);
        this.cachedTextArea = new JTextArea(20, 20);
        searchScrollPane = new JScrollPane(searchResults);
        searchResults.setLineWrap(true);
        searchResults.setWrapStyleWord(true);

        JButton spellIt = new JButton("Spell");
        JButton search = new JButton("GoogleSearch");
        JButton cachedButton = new JButton("Cached Page");
        search.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    search();
                }
            }
        );
        spellIt.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    getSpelling();
                }
            }
        );
        cachedButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    getCachedPage();
                }
            }
        );

        JPanel spellPanel = new JPanel();
        JPanel searchPanel = new JPanel();
        JPanel cachedPanel = new JPanel();
        spellPanel.add(new JLabel("Spell: "));
        spellPanel.add(this.spelling);
        spellPanel.add(spellIt);
        this.tempLabel = new JLabel("             ");
        spellPanel.add(new JLabel("Correct Spelling: "));
        spellPanel.add(this.tempLabel);
        searchPanel.setLayout(new BorderLayout());

        JPanel tempPanel = new JPanel();
        tempPanel.add(searchString);
        tempPanel.add(search);
        searchPanel.add(tempPanel, BorderLayout.NORTH);

        //searchPanel.add(search,BorderLayout.CENTER);
        searchPanel.add(searchScrollPane, BorderLayout.SOUTH);
        cachedPanel.add(new JLabel("Enter a URL:"));
        cachedPanel.add(cachedURL);
        cachedPanel.add(cachedButton);
        cachedPanel.add(cachedTextArea);
        setLayout(new BorderLayout());
        add(spellPanel, BorderLayout.NORTH);
        add(searchPanel, BorderLayout.CENTER);
        add(cachedPanel, BorderLayout.SOUTH);
    }

    private void getSpelling() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        String tempSpell = this.spelling.getText();
        GoogleSearchServiceLocator googleService =
            new GoogleSearchServiceLocator();

        try {
            GoogleSearchPort google =
                googleService.getGoogleSearchPort(this.defaultEndpoint);
            String temp = google.doSpellingSuggestion(key, tempSpell);
            this.tempLabel.setText(temp);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Unable to get Spelling " + e.getMessage(),
                "GoogleSearch: Spell error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private void search() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        String searchStringText = this.searchString.getText();
        this.searchResults.setText("");

        GoogleSearchServiceLocator googleService =
            new GoogleSearchServiceLocator();

        try {
            GoogleSearchPort google =
                googleService.getGoogleSearchPort(this.defaultEndpoint);
            System.out.println(
                "key: " + key + " search String : " + searchStringText
            );

            GoogleSearchResult tempResults =
                google.doGoogleSearch(
                    key, searchStringText, 0, 10, true, "string", false,
                    "string", "latin1", "latin1"
                );
            displayResults(tempResults);

        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Unable to Search  " + e.getMessage(),
                "GoogleSearch: Search error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private void getCachedPage() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        String cachedURLText = this.cachedURL.getText();
        GoogleSearchServiceLocator googleService =
            new GoogleSearchServiceLocator();

        try {
            GoogleSearchPort google =
                googleService.getGoogleSearchPort(this.defaultEndpoint);
            byte[] cachedBytes = google.doGetCachedPage(key, cachedURLText);
            String cachedString = new String(cachedBytes);
            this.cachedTextArea.append(cachedString);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Unable to Search  " + e.getMessage(),
                "GoogleSearch: Search error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private void displayResults(GoogleSearchResult results) {
        if (results == null) {
            return;
        }

        StringWriter writer = new StringWriter();
        PrintWriter out = new PrintWriter(writer);
        out.println("Comment: " + results.getSearchComments());
        out.println("Query: " + results.getSearchQuery());

        ResultElement[] resultElements = results.getResultElements();

        for (int i = 0; i < resultElements.length; i++) {
            out.println("  Hit " + i + " :");
            out.println("    URL: " + resultElements[i].getURL());
            out.println("    Summary: " + resultElements[i].getSummary());
            out.println("    Title: " + resultElements[i].getTitle());
            out.println("    Host: " + resultElements[i].getHostName());
            out.println("    Snippet: " + resultElements[i].getSnippet());
        }

        out.flush();
        this.searchResults.append(writer.toString());
    }
}
