/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.NotificationSinkCallback;
import org.globus.ogsa.ServiceData;
import org.globus.ogsa.client.managers.NotificationSinkManager;
import org.globus.ogsa.types.query.ServiceDataXPathQueryExpressionType;
import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.ExtendedDateTimeHelper;
import org.globus.ogsa.utils.GSIUtils;
import org.globus.ogsa.utils.QueryHelper;
import org.globus.ogsa.utils.XmlFactory;

import org.gridforum.ogsi.ExtendedDateTimeType;
import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.GridService;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.FaultType;
import org.gridforum.ogsi.TargetInvalidFaultType;
import org.gridforum.ogsi.InfinityType;
import org.gridforum.ogsi.OGSIServiceGridLocator;
import org.gridforum.ogsi.ServiceDataValuesType;
import org.gridforum.ogsi.TerminationTimeType;

import org.w3c.dom.Element;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.rmi.RemoteException;

import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;

public class GridServicePortTypePanel extends AbstractPortTypePanel
    implements NotificationSinkCallback {
    private JTextField serviceDataNamespaceTF;
    private JTextField serviceDataNameTF;
    private JTextField timeoutTF;
    private JTextField subscriptionTimeoutTF;
    private JTextField xpathExprTF;
    private JTextField xpathNamespacesTF;
    private JLabel subscriptionTimeoutLB;
    private JButton subscribeNameBT;
    private JButton unsubscribeNameBT;
    private JTextField dateTimeTF;
    private XMLTree xmlTree;
    private OGSIServiceGridLocator gridServiceLocator =
        new OGSIServiceGridLocator();
    private Hashtable subscriptions = new Hashtable();
    private Hashtable properties = new Hashtable();
    private NotificationSinkManager manager = null;

    public GridServicePortTypePanel() {
        super(" Grid Service ");

        try {
            this.xmlTree = new XMLTree(XmlFactory.newDocument());
        } catch (Exception e) {
            e.printStackTrace();
        }

        this.serviceDataNamespaceTF = new JTextField(20);
        this.serviceDataNameTF = new JTextField(20);
        this.subscriptionTimeoutTF = new JTextField(20);
        this.xpathExprTF = new JTextField(20);
        this.xpathNamespacesTF = new JTextField(30);

        JButton queryNameBT = new JButton("Query");
        this.subscribeNameBT = new JButton("Subscribe");
        this.subscribeNameBT.setVisible(false);
        this.unsubscribeNameBT = new JButton("Unsubscribe");
        this.unsubscribeNameBT.setVisible(false);

        queryNameBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        QName qname =
                            new QName(
                                serviceDataNamespaceTF.getText(),
                                serviceDataNameTF.getText()
                            );

                        if (xpathExprTF.getText().length() == 0) {
                            doQuery(QueryHelper.getNamesQuery(qname));
                        } else {
                            String[] namespaces =
                                QueryHelper.tokenizeNamespaces(
                                    xpathNamespacesTF.getText()
                                );
                            doQuery(
                                QueryHelper.getXPathQuery(
                                    qname, xpathExprTF.getText(), namespaces
                                )
                            );
                        }
                    } catch (Exception exc) {
                        exc.printStackTrace();
                    }
                }
            }
        );

        this.subscribeNameBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    QName qname =
                        new QName(
                            serviceDataNamespaceTF.getText(),
                            serviceDataNameTF.getText()
                        );
                    doSubscribe(qname);
                }
            }
        );

        this.unsubscribeNameBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    QName qname =
                        new QName(
                            serviceDataNamespaceTF.getText(),
                            serviceDataNameTF.getText()
                        );
                    doUnsubscribe(qname);
                }
            }
        );

        JPanel queryPanel = new JPanel();
        JPanel namePanel = new JPanel();
        JPanel namePanel2 = new JPanel();
        JPanel buttonPanel = new JPanel();

        namePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        namePanel2.setLayout(new FlowLayout(FlowLayout.LEFT));
        buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        queryPanel.setLayout(new BoxLayout(queryPanel, BoxLayout.Y_AXIS));

        namePanel.add(new JLabel("Namespace: "));
        namePanel.add(this.serviceDataNamespaceTF);
        namePanel.add(new JLabel("Name: "));
        namePanel.add(this.serviceDataNameTF);
        this.subscriptionTimeoutLB = new JLabel("Timeout: ");
        namePanel.add(this.subscriptionTimeoutLB);
        namePanel.add(this.subscriptionTimeoutTF);
        this.subscriptionTimeoutLB.setVisible(false);
        this.subscriptionTimeoutTF.setVisible(false);
        namePanel2.add(new JLabel("XPath Expression: "));
        namePanel2.add(this.xpathExprTF);
        namePanel2.add(new JLabel("XPath Namespace Mappings: "));
        namePanel2.add(this.xpathNamespacesTF);

        buttonPanel.add(queryNameBT);
        buttonPanel.add(this.subscribeNameBT);
        buttonPanel.add(this.unsubscribeNameBT);
        queryPanel.add(namePanel);
        queryPanel.add(namePanel2);
        queryPanel.add(buttonPanel);

        this.timeoutTF = new JTextField(10);
        this.dateTimeTF = new JTextField(25);

        JButton setTimeoutBT = new JButton("Set");
        JButton getTimeoutBT = new JButton("Get");
        JButton destroyBT = new JButton("Destroy");

        JPanel destroyPanel = new JPanel();
        destroyPanel.add(new JLabel("Timeout(seconds): "));
        destroyPanel.add(this.timeoutTF);
        destroyPanel.add(this.dateTimeTF);
        this.dateTimeTF.setEditable(false);
        destroyPanel.add(setTimeoutBT);
        destroyPanel.add(getTimeoutBT);
        destroyPanel.add(destroyBT);

        setTimeoutBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setTimeout();
                }
            }
        );
        getTimeoutBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    getTimeout();
                }
            }
        );
        destroyBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    destroy();
                }
            }
        );

        setLayout(new BorderLayout());
        add(queryPanel, BorderLayout.NORTH);
        add(new JScrollPane(xmlTree), BorderLayout.CENTER);
        add(destroyPanel, BorderLayout.SOUTH);
    }

    private void doQuery(ExtensibilityType queryExpression) {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            GridService service =
                this.gridServiceLocator.getGridServicePort(
                    this.defaultEndpoint
                );
            GSIUtils.setDefaultGSIProperties(
                (Stub) service, this.defaultEndpoint
            );
            this.context.setAuthentication((Stub) service);

            ExtensibilityType queryResult =
                service.findServiceData(queryExpression);

            if (queryResult.get_any() == null) {
                // a null any in the case of an xpath query means 0 results
                Object obj = AnyHelper.getAsSingleObject(queryExpression);

                if (obj instanceof ServiceDataXPathQueryExpressionType) {
                    JOptionPane.showMessageDialog(
                        this, "XPath Query: No results found",
                        "GridService: XPath Query",
                        JOptionPane.INFORMATION_MESSAGE
                    );

                    return;
                }
            }

            Element result = AnyHelper.getAsParentElement(queryResult);
            this.xmlTree.setModel(new XMLTreeModel(result));
        } catch (TargetInvalidFaultType fault) {
            String desc = getDescription(fault);
            JOptionPane.showMessageDialog(
                this, "Service Data was not found: " + desc,
                "GridService: query error", JOptionPane.ERROR_MESSAGE
            );
        } catch (FaultType fault) {
            String desc = getDescription(fault);
            JOptionPane.showMessageDialog(
                this, "Failed to perform query: " + desc,
                "GridService: query error", JOptionPane.ERROR_MESSAGE
            );
            fault.printStackTrace();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to perform query: " + e.getMessage(),
                "GridService: query error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private String getDescription(FaultType fault) {
        String[] descs = fault.getDescription();
        String desc = "";
        for (int i = 0; (descs != null) && (i < descs.length); i++) {
            desc = desc + "[" + descs[i] + "]";
        }
        return desc;
    }

    private void doSubscribe(QName qname) {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            Calendar timeout = Calendar.getInstance();
            String timeoutSeconds = this.subscriptionTimeoutTF.getText().trim();

            if ((timeoutSeconds == null) || timeoutSeconds.equals("")) {
                throw new Exception("Subscription timeout not set.");
            }

            timeout.add(Calendar.SECOND, Integer.parseInt(timeoutSeconds));

            String subscriptionID =
                this.manager.addListener(
                    qname, timeout, new HandleType(this.handle.toString()), this
                );
            this.subscriptions.put(qname, subscriptionID);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to perform subscription request: " + e.getMessage(),
                "GridService: subscription error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private void doUnsubscribe(QName qname) {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            String subscriptionID = (String) this.subscriptions.get(qname);
            this.manager.removeListener(subscriptionID);
            this.subscriptions.remove(qname);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to perform unsubscribe request: " +
                e.getMessage(), "GridService: unsubscribe error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private void setTimeout() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

	if (this.timeoutTF.getText().trim().length() == 0) {
	    return;
	}

        try {
            GridService service =
                gridServiceLocator.getGridServicePort(this.defaultEndpoint);
            GSIUtils.setDefaultGSIProperties(
                (Stub) service, this.defaultEndpoint
            );
            this.context.setAuthentication((Stub) service);

            ExtendedDateTimeType dateTime = new ExtendedDateTimeType();

            if (
                this.timeoutTF.getText().equals(
                        InfinityType.infinity.toString()
                    )
            ) {
                dateTime.setValue(InfinityType.infinity);
            } else {
                Calendar timeout = Calendar.getInstance();
                timeout.add(
                    Calendar.SECOND, Integer.parseInt(this.timeoutTF.getText())
                );
                dateTime.setValue(timeout);
            }

            TerminationTimeType serviceTime =
                service.requestTerminationAfter(dateTime);

            if (ExtendedDateTimeHelper.isInfinity(serviceTime.getAfter())) {
                this.dateTimeTF.setText(InfinityType.infinity.toString());
            } else {
                Calendar serviceTimeout =
                    serviceTime.getAfter().getDateTimeValue();
                long seconds =
                    (
                        serviceTimeout.getTime().getTime() -
                        new Date().getTime()
                    ) / 1000;
                this.timeoutTF.setText(String.valueOf(seconds));
                this.dateTimeTF.setText(
                    serviceTime.getAfter().getDateTimeValue().getTime()
                               .toString()
                );
            }
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
		this, "Failed to set termination time: " + getErrorMessage(e),
                "GridService: setTimeout error", JOptionPane.ERROR_MESSAGE
            );
        }
    }

    private void getTimeout() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            GridService service =
                this.gridServiceLocator.getGridServicePort(
                    this.defaultEndpoint
                );
            GSIUtils.setDefaultGSIProperties(
                (Stub) service, this.defaultEndpoint
            );
            this.context.setAuthentication((Stub) service);

            ExtensibilityType queryResult =
                service.findServiceData(
                    QueryHelper.getNamesQuery(ServiceData.TERMINATION)
                );
	    ServiceDataValuesType serviceDataValues =
		(ServiceDataValuesType) AnyHelper.getAsSingleObject(
		     queryResult, ServiceDataValuesType.class
		);
            TerminationTimeType terminationTime =
                (TerminationTimeType) AnyHelper.getAsSingleObject(
                    serviceDataValues, TerminationTimeType.class
                );

            if (ExtendedDateTimeHelper.isInfinity(terminationTime)) {
                this.dateTimeTF.setText(InfinityType.infinity.toString());
            } else {
                Calendar serviceTimeout =
                    terminationTime.getAfter().getDateTimeValue();
                long seconds =
                    (
                        serviceTimeout.getTime().getTime() -
                        new Date().getTime()
                    ) / 1000;
                this.timeoutTF.setText(String.valueOf(seconds));
                this.dateTimeTF.setText(serviceTimeout.getTime().toString());
            }
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
  	        this, "Failed to get termination time: " + getErrorMessage(e),
                "GridService: getTimeout error", JOptionPane.ERROR_MESSAGE
            );
        }
    }

    private void destroy() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        try {
            GridService service =
                gridServiceLocator.getGridServicePort(this.defaultEndpoint);
            GSIUtils.setDefaultGSIProperties(
                (Stub) service, this.defaultEndpoint
            );
            this.context.setAuthentication((Stub) service);
            service.destroy();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to destroy service: " +  getErrorMessage(e),
                "GridService: destroy error", JOptionPane.ERROR_MESSAGE
            );
        }
    }

    public void init() {
        checkNotificationSupport();
    }

    public void dispose() {
        for (
            Enumeration enum = this.subscriptions.elements();
                enum.hasMoreElements();
        ) {
            String subscriptionID = (String) enum.nextElement();

            try {
                this.manager.removeListener(subscriptionID);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public Object getProperty(String name) {
        return this.properties.get(name);
    }

    public Object getPersistentProperty(String name) {
        return null;
    }

    public void setProperty(
        String name,
        Object property
    ) {
        this.properties.put(name, property);
    }

    public void setPersistentProperty(
        String name,
        Object property
    ) {
    }

    public void flush() {
    }

    public void deliverNotification(ExtensibilityType message)
        throws RemoteException {
        try {
            Element element = (Element) AnyHelper.getAsSingleElement(message);
            this.xmlTree.setModel(new XMLTreeModel(element));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void checkNotificationSupport() {
        try {
            GridService service =
                gridServiceLocator.getGridServicePort(this.defaultEndpoint);
            GSIUtils.setDefaultGSIProperties(
                (Stub) service, this.defaultEndpoint
            );

            ExtensibilityType any =
                service.findServiceData(
                    QueryHelper.getNamesQuery(
                        ServiceData.SUBSCRIBE_EXTENSIBILITY
                    )
                );
            ServiceDataValuesType serviceDataValues =
                AnyHelper.getAsServiceDataValues(any);

            if (serviceDataValues.get_any() == null) {
                return;
            }

            this.manager = NotificationSinkManager.getManager();
            this.manager.startListening(NotificationSinkManager.CHILD_THREAD);
            this.subscriptionTimeoutTF.setVisible(true);
            this.subscriptionTimeoutLB.setVisible(true);
            this.subscribeNameBT.setVisible(true);
            this.unsubscribeNameBT.setVisible(true);
        } catch (Exception e) {
            // e.printStackTrace();
        }
    }
}
