/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.impl.security.Constants;
import org.globus.ogsa.impl.security.authorization.SelfAuthorization;
import org.globus.ogsa.NotificationSinkCallback;
import org.globus.ogsa.client.managers.NotificationSinkManager;
import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.XmlFactory;

import org.globus.axis.gsi.GSIConstants;

import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.HandleType;

import org.w3c.dom.Element;

import javax.swing.JScrollPane;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

import java.rmi.RemoteException;

import java.util.Hashtable;


public class NotificationSinkPortTypePanel extends AbstractPortTypePanel
    implements NotificationSinkCallback {
    private Hashtable properties = new Hashtable();
    private NotificationSinkManager manager;
    private String sink;
    private XMLTree xmlTree;

    public NotificationSinkPortTypePanel() {
        super(" Notification Sink ");

        setLayout(new BorderLayout());

        try {
            this.xmlTree = new XMLTree(XmlFactory.newDocument());
        } catch (Exception e) {
            e.printStackTrace();
        }

        JScrollPane scrollPane = new JScrollPane(this.xmlTree);
        scrollPane.setPreferredSize(new Dimension(200, 150));
        add(scrollPane, BorderLayout.CENTER);
    }

    public void init() {
        // for notification: sets authorization type
        setProperty(Constants.AUTHORIZATION, SelfAuthorization.getInstance());

        // sets gsi mode
        setProperty(GSIConstants.GSI_MODE, GSIConstants.GSI_MODE_NO_DELEG);

        addTopicListener();
    }

    public void dispose() {
        try {
            if (this.manager.isListening()) {
                this.manager.removeListener(this.sink);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void addTopicListener() {
        try {
            this.manager = NotificationSinkManager.getManager();
            this.manager.startListening(NotificationSinkManager.CHILD_THREAD);
            this.sink =
                this.manager.addListener(
                    "MessageGateway", null,
                    new HandleType(this.handle.toString()), this
                );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Object getProperty(String name) {
        return this.properties.get(name);
    }

    public Object getPersistentProperty(String name) {
        return null;
    }

    public void setProperty(
        String name,
        Object property
    ) {
        this.properties.put(name, property);
    }

    public void setPersistentProperty(
        String name,
        Object property
    ) {
    }

    public void flush() {
    }

    public void deliverNotification(ExtensibilityType message)
        throws RemoteException {
        try {
            Element element = AnyHelper.getAsSingleElement(message);
            this.xmlTree.setModel(new XMLTreeModel(element));

            Color color = this.xmlTree.getBackground();
            this.xmlTree.setBackground(Color.red);
            Thread.currentThread().sleep(200);
            this.xmlTree.setBackground(color);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
