/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.impl.core.logging.OgsiLogging;
import org.globus.ogsa.NotificationSinkCallback;
import org.globus.ogsa.client.managers.NotificationSinkManager;
import org.globus.ogsa.core.logging.GroupLogAttribute;
import org.globus.ogsa.core.logging.LogAttributeElement;
import org.globus.ogsa.core.logging.LogMessageElement;
import org.globus.ogsa.core.logging.OgsiLoggingPortType;
import org.globus.ogsa.core.logging.service.OgsiLoggingServiceLocator;
import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.QueryHelper;
import org.globus.ogsa.impl.security.authorization.NoAuthorization;
import org.globus.ogsa.impl.security.Constants;

import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.FaultType;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.ServiceDataValuesType;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.rmi.RemoteException;

import java.util.Hashtable;
import java.util.StringTokenizer;


public class OgsiLoggingPortTypePanel extends AbstractPortTypePanel
    implements NotificationSinkCallback {
    private OgsiLoggingServiceLocator locator;
    private JComboBox attributeCB;
    private JComboBox persistentCB;
    private JTextField attributeNameTF;
    private JTextField destinationTF;
    private JTextField levelTF;
    private JTextField groupTF;
    private JButton setTransientAttributeBT;
    private JButton setPersistentAttributeBT;
    private JButton changeGroupTransientAttributeBT;
    private JButton changeGroupPersistentAttributeBT;
    private JLabel nameLabel;
    private JScrollPane logMessagesPane;
    private JRadioButton attributeOps;
    private JRadioButton groupOps;
    private JTextArea outputTextArea;
    private JButton loadPersistentAttributesBT;
    private JButton saveTransientAttributesBT;
    private JButton logSizeBT;
    private JTextField sizeTF;
    private NotificationSinkManager manager;
    private String sink;
    private Hashtable properties = new Hashtable();

    /***********************************************************************
     * <code>OgsiLoggingPortTypePanel</code>
     *
     * <p>Panel to display the OgsiLoggingPortType with the service browser.</p>
     *
     * @author <a href="mailto:wley@us.ibm.com">John Wiley</a>
     * @author John Wiley
     * @author OGSA Development, IBM Poughkeepsie
     ***************************************************************************************************/
    public OgsiLoggingPortTypePanel() {
        super(" OGSI Logging portType ");
        this.locator = new OgsiLoggingServiceLocator();

        // Set up required panels.  
        JPanel servicePanel = new JPanel();
        servicePanel.setLayout(new BoxLayout(servicePanel, BoxLayout.Y_AXIS));

        JPanel attributePanel = new JPanel();
        attributePanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel persistentPanel = new JPanel();
        persistentPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel buttonPanel1 = new JPanel();
        buttonPanel1.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel buttonPanel2 = new JPanel();
        buttonPanel2.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel buttonPanel3 = new JPanel();
        buttonPanel3.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel serviceDataPanel = new JPanel();
        serviceDataPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel logPanel = new JPanel();
        logPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        // Add the Transient Attribute combo box to a panel
        attributePanel.add(new JLabel("Transient Attributes: "));
        this.attributeCB = new JComboBox();
        attributeCB.setEditable(false);
        attributePanel.add(this.attributeCB);

        // Add the Persistent Attribute combo box to a panel
        persistentPanel.add(new JLabel("Persistent Attributes: "));
        this.persistentCB = new JComboBox();
        persistentCB.setEditable(false);
        persistentPanel.add(this.persistentCB);

        // Add the buttons to another panel
        setTransientAttributeBT = new JButton("setTransientAttribute");
        setPersistentAttributeBT = new JButton("setPersistentAttribute");
        changeGroupTransientAttributeBT =
            new JButton("changeGroupTransientAttribute");
        changeGroupPersistentAttributeBT =
            new JButton("changeGroupPersistentAttribute");
        attributeOps = new JRadioButton("attributes");
        groupOps = new JRadioButton("groups");

        loadPersistentAttributesBT = new JButton("loadPersistentAttributes");
        saveTransientAttributesBT = new JButton("saveTransientAttributes");

        buttonPanel1.add(attributeOps);
        buttonPanel2.add(groupOps);

        buttonPanel1.add(setTransientAttributeBT);
        buttonPanel1.add(setPersistentAttributeBT);
        buttonPanel2.add(changeGroupTransientAttributeBT);
        buttonPanel2.add(changeGroupPersistentAttributeBT);

        buttonPanel3.add(saveTransientAttributesBT);
        buttonPanel3.add(loadPersistentAttributesBT);

        setTransientAttributeBT.setEnabled(true);
        setPersistentAttributeBT.setEnabled(true);
        changeGroupTransientAttributeBT.setEnabled(false);
        changeGroupPersistentAttributeBT.setEnabled(false);

        //defaults at start
        attributeOps.setSelected(true);

        groupOps.setSelected(false);

        groupOps.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setTransientAttributeBT.setEnabled(false);
                    setPersistentAttributeBT.setEnabled(false);
                    changeGroupTransientAttributeBT.setEnabled(true);
                    changeGroupPersistentAttributeBT.setEnabled(true);
                    attributeNameTF.setEditable(false);
                    attributeOps.setSelected(false);
                    groupOps.setSelected(true);
                }
            }
        );

        attributeOps.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setTransientAttributeBT.setEnabled(true);
                    setPersistentAttributeBT.setEnabled(true);
                    changeGroupTransientAttributeBT.setEnabled(false);
                    changeGroupPersistentAttributeBT.setEnabled(false);
                    attributeNameTF.setEditable(true);
                    attributeOps.setSelected(true);
                    groupOps.setSelected(false);
                }
            }
        );

        setTransientAttributeBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setTransient();
                    refreshAts();
                    refreshCon();
                    refreshSize();
                }
            }
        ); //changeGroupTransientAttribute

        setPersistentAttributeBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setPersistent();
                    refreshCon();
                    refreshAts();
                    refreshSize();
                }
            }
        );

        changeGroupTransientAttributeBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setGroupTransient();
                    refreshAts();
                    refreshCon();
                    refreshSize();
                }
            }
        );

        changeGroupPersistentAttributeBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setGroupPersistent();
                    refreshCon();
                    refreshAts();
                    refreshSize();
                }
            }
        );

        loadPersistentAttributesBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    loadPersistent();
                    refreshAts();
                    refreshCon();
                    refreshSize();
                }
            }
        );

        saveTransientAttributesBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    saveTransient();
                    refreshAts();
                    refreshCon();
                    refreshSize();
                }
            }
        );

        // Set up service data labels and text areas
        nameLabel = new JLabel("name: ");
        attributeNameTF = new JTextField(40);
        attributeNameTF.setEditable(true);

        JLabel destinationLabel = new JLabel("destination: ");
        destinationTF = new JTextField(5);
        destinationTF.setEditable(true);

        JLabel levelLabel = new JLabel("level: ");
        levelTF = new JTextField(5);
        levelTF.setEditable(true);

        JLabel groupLabel = new JLabel("group: ");
        groupTF = new JTextField(10);
        groupTF.setEditable(true);

        serviceDataPanel.add(nameLabel);
        serviceDataPanel.add(attributeNameTF);
        serviceDataPanel.add(levelLabel);
        serviceDataPanel.add(levelTF);
        serviceDataPanel.add(destinationLabel);
        serviceDataPanel.add(destinationTF);
        serviceDataPanel.add(groupLabel);
        serviceDataPanel.add(groupTF);

        // Setup log panel
        this.outputTextArea = new JTextArea();
        logMessagesPane = new JScrollPane(this.outputTextArea);
        logMessagesPane.setPreferredSize(new Dimension(600, 225));

        JLabel logLabel = new JLabel("Log Messages View ");

        //logPanel.add(logLabel);
        logPanel.add(logMessagesPane, BorderLayout.CENTER);
        this.outputTextArea.setEditable(false);

        logSizeBT = new JButton("setLogBufferSize");
        sizeTF = new JTextField(5);
        sizeTF.setEditable(true);

        logPanel.add(logSizeBT);
        logPanel.add(sizeTF);

        logSizeBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setLogSize();
                    refreshSize();
                    refreshLog();
                }
            }
        );

        persistentCB.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setConFields();
                }
            }
        );

        attributeCB.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setAtsFields();
                }
            }
        );

        // Add the attribute, attributeData, buttons and log view panels to
        // the servicePanel that is using the BoxLayout Mgr
        servicePanel.add(attributePanel);
        servicePanel.add(persistentPanel);
        servicePanel.add(serviceDataPanel);
        servicePanel.add(buttonPanel1);
        servicePanel.add(buttonPanel2);
        servicePanel.add(logLabel);
        servicePanel.add(logPanel);
        servicePanel.add(buttonPanel3);

        // Add the completed Service Panel
        setLayout(new BorderLayout());
        add(servicePanel, BorderLayout.NORTH);
    }

    /**
    * <code>setLogSize</code>
    *
    * Process the setLogSize button.
    *
    */
    private void setLogSize() {
        try {
            //System.out.println("setLogSize");
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            OgsiLoggingPortType loggingPT =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingPT);

            loggingPT.setLogBufferSize(Integer.parseInt(sizeTF.getText()));
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to setLogSize: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: setLogSize error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to do something: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: setLogSize error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    //populate input text with selected item data 
    private void setConFields() {
        String line = (String) persistentCB.getSelectedItem();

        if (line != null) {
            if (!line.equals(null)) {
                StringTokenizer st = new StringTokenizer(line);

                attributeNameTF.setText(st.nextToken());

                String lev = st.nextToken();
                lev = lev.substring(1);
                levelTF.setText(lev);

                String grp = st.nextToken();
                grp = grp.substring(1, grp.length() - 1);

                if (grp.equals("-")) {
                    grp = " ";
                }

                groupTF.setText(grp);

                destinationTF.setText(st.nextToken());
            }
        }
    }

    //populate input text with selected item data 
    private void setAtsFields() {
        String line = (String) attributeCB.getSelectedItem();

        if (line != null) {
            if (!line.equals(null)) {
                StringTokenizer st = new StringTokenizer(line);

                attributeNameTF.setText(st.nextToken());

                String lev = st.nextToken();
                lev = lev.substring(1);
                levelTF.setText(lev);

                String grp = st.nextToken();
                grp = grp.substring(1, grp.length() - 1);

                if (grp.equals("-")) {
                    grp = " ";
                }

                groupTF.setText(grp);

                destinationTF.setText(st.nextToken());
            }
        }
    }

    /**
    * <code>loadPersistent</code>
    *
    * Process the loadPersistent button.
    *
    */
    private void loadPersistent() {
        try {
            //System.out.println("loadPersistent");
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            OgsiLoggingPortType loggingPT =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingPT);

            loggingPT.loadPersistentAttributes();
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to loadPersistent: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: loadPersistent error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to do something: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: loadPersistent error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
    * <code>saveTransient</code>
    *
    * Process the saveTransient button.
    *
    */
    private void saveTransient() {
        try {
            //System.out.println("saveTransient");
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            OgsiLoggingPortType loggingPT =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingPT);

            loggingPT.saveTransientAttributes();
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to saveTransient: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: saveTransient error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to do something: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: saveTransient error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>setTransient</code>
     *
     * Process the setTransient button.
     *
     */
    private void setTransient() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            OgsiLoggingPortType loggingPT =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingPT);

            loggingPT.setTransientAttribute(makeLogA());
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to setTransient: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: setTransient error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to do something: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: setTransient error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>setPersistent</code>
     *
     * Process the setPersistent button.
     *
     */
    private void setPersistent() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            OgsiLoggingPortType loggingPT =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingPT);

            loggingPT.setPersistentAttribute(makeLogA());
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to setPersistent: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: setPersistent error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to setPersistent: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: setPersistent error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>setGroupTransient</code>
     *
     * Process the setGroupTransient button.
     *
     */
    private void setGroupTransient() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            OgsiLoggingPortType loggingPT =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingPT);

            loggingPT.changeGroupTransientAttribute(makeGrpA());
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to setGroupTransient: " + ft.getClass().getName() +
                "\n" + ft.getDescription()[0],
                "OGSI Logging: setGroupTransient error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to setGroupTransient: " + e.getClass().getName() +
                "\n" + e.getMessage(), "OGSI Logging: setGroupTransient error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>setGroupPersistent</code>
     *
     * Process the setGroupPersistent button.
     *
     */
    private void setGroupPersistent() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            OgsiLoggingPortType loggingPT =
                locator.getOgsiLoggingPort(this.defaultEndpoint);
	    
	    this.context.setAuthentication((Stub)loggingPT);

            loggingPT.changeGroupPersistentAttribute(makeGrpA());
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to setGroupPersistent: " + ft.getClass().getName() +
                "\n" + ft.getDescription()[0],
                "OGSI Logging: setGroupPersistent error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to setGroupPersistent: " + e.getClass().getName() +
                "\n" + e.getMessage(), "OGSI Logging: setGroupPersistent error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     *
     * <code>refreshAttributecombo</code>
     *
     * Get the TransientAttributes service data element.
     *
     */
    private void refreshAts() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

	    OgsiLoggingPortType loggingService =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingService);

            // Get the transient attributes service data element
            ExtensibilityType queryResult =
                loggingService.findServiceData(
                    QueryHelper.getNamesQuery(
                        new QName[] { OgsiLogging.TRANSIENT_ATTRIBUTES }
                    )
                );

            if (queryResult == null) {
                this.attributeCB.removeAllItems();
                this.attributeCB.addItem(" no runtime records found");

                return;
            }

            if (
                (queryResult.get_any() != null) &&
                    (queryResult.get_any().length > 0)
            ) {
                ServiceDataValuesType serviceDataValues =
                    (ServiceDataValuesType) AnyHelper.getAsSingleObject(
                        queryResult, ServiceDataValuesType.class
                    );

                if (
                    (serviceDataValues == null) ||
                        serviceDataValues.equals(null) ||
                        (serviceDataValues.get_any() == null)
                ) {
                    this.attributeCB.removeAllItems();
                    this.attributeCB.addItem(" no runtime records found");

                    return;
                }

                Object[] loga =
                    AnyHelper.getAsObject(
                        serviceDataValues, LogAttributeElement.class
                    );

                if (loga == null) {
                    this.attributeCB.removeAllItems();
                    this.attributeCB.addItem(" no runtime records found");

                    return;
                }

                // Refresh the list in the combox box.
                this.attributeCB.removeAllItems();

                for (int i = 0; i < loga.length; i++) {
                    try {
                        LogAttributeElement logE =
                            (LogAttributeElement) loga[i];
                        this.attributeCB.addItem(getAttributeLine(logE));
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: refresh ats error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh ats: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: refresh ats error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    //display composition helper logAttributeElement lines
    private String getAttributeLine(LogAttributeElement logAe) {
        String grp = logAe.getGroupName();

        if (grp != null) {
            if (!grp.equals(null)) {
                ;
            } else {
                grp = "-";
            }
        } else {
            grp = "-";
        }

        return (logAe.getAttributeName() + "    :" + logAe.getLevel() + "   (" +
        grp + ")   " + logAe.getDestination());
    }

    /**
     * routine to convert name, dest, level, group to a Logattribute
     */
    private LogAttributeElement makeLogA() {
        LogAttributeElement loga = new LogAttributeElement();

        loga.setAttributeName(attributeNameTF.getText().trim());
        loga.setDestination(destinationTF.getText().trim());
        loga.setLevel(levelTF.getText().trim());
        loga.setGroupName(groupTF.getText().trim());

        return loga;
    }

    /**
     * routine to convert name, dest, level, group to a Logattribute
     */
    private GroupLogAttribute makeGrpA() {
        GroupLogAttribute loga = new GroupLogAttribute();

        loga.setDestination(destinationTF.getText());
        loga.setLevel(levelTF.getText().trim());

        String grp = groupTF.getText();

        if (grp != null) {
            if (!grp.equals("") && !grp.equals(" ")) {
                loga.setGroup(groupTF.getText());
            }
        }

        return loga;
    }

    /**
     *
     * <code>refreshPersistent combo</code>
     *
     * Get the PersistentAttributes service data element.
     *
     */
    private void refreshCon() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

	    OgsiLoggingPortType loggingService =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingService);

            // Get the transient attributes service data element
            ExtensibilityType queryResult =
                loggingService.findServiceData(
                    QueryHelper.getNamesQuery(
                        new QName[] { OgsiLogging.PERSISTENT_ATTRIBUTES }
                    )
                );

            if (queryResult == null) {
                this.persistentCB.removeAllItems();
                this.persistentCB.addItem(" no persistent records found");

                return;
            }

            if (
                (queryResult.get_any() != null) &&
                    (queryResult.get_any().length > 0)
            ) {
                ServiceDataValuesType serviceDataValues =
                    (ServiceDataValuesType) AnyHelper.getAsSingleObject(
                        queryResult, ServiceDataValuesType.class
                    );

                if (
                    (serviceDataValues == null) ||
                        serviceDataValues.equals(null) ||
                        (serviceDataValues.get_any() == null)
                ) {
                    this.persistentCB.removeAllItems();
                    this.persistentCB.addItem(" no persistent records found");

                    return;
                }

                Object[] loga =
                    AnyHelper.getAsObject(
                        serviceDataValues, LogAttributeElement.class
                    );

                if (loga == null) {
                    this.persistentCB.removeAllItems();
                    this.persistentCB.addItem(" no persistent records found");

                    return;
                }

                // Refresh the list in the combox box.
                this.persistentCB.removeAllItems();

                for (int i = 0; i < loga.length; i++) {
                    try {
                        LogAttributeElement logE =
                            (LogAttributeElement) loga[i];
                        this.persistentCB.addItem(getAttributeLine(logE));
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: refresh Con error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh con: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: refresh con error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     *
     * <code>refreshAttributecombo</code>
     *
     * Get the service counts service data element.
     *
     */
    private void refreshLog() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

	    OgsiLoggingPortType loggingService =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingService);

            // Get the transient attributes service data element
            ExtensibilityType queryResult =
                loggingService.findServiceData(
                    QueryHelper.getNamesQuery(
                        new QName[] { OgsiLogging.LOG_MESSAGES }
                    )
                );

            if (queryResult == null) {
                this.outputTextArea.setText(null);
                this.outputTextArea.append(" no messages logged \n");

                return;
            }

            if (
                (queryResult.get_any() != null) &&
                    (queryResult.get_any().length > 0)
            ) {
                ServiceDataValuesType serviceDataValues =
                    (ServiceDataValuesType) AnyHelper.getAsSingleObject(
                        queryResult, ServiceDataValuesType.class
                    );

                if (
                    (serviceDataValues == null) ||
                        serviceDataValues.equals(null) ||
                        (serviceDataValues.get_any() == null)
                ) {
                    this.outputTextArea.setText(null);
                    this.outputTextArea.append(" no messages logged \n");

                    return;
                }

                Object[] logm =
                    AnyHelper.getAsObject(
                        serviceDataValues, LogMessageElement.class
                    );

                if (logm == null) {
                    this.outputTextArea.setText(null);
                    this.outputTextArea.append(" no messages logged \n");

                    return;
                }

                // clear the log view
                this.outputTextArea.setText(null);

                //repopulate
                for (int i = 0; i < logm.length; i++) {
                    try {
                        LogMessageElement logMe = (LogMessageElement) logm[i];
                        String lev = logMe.getLevel().trim();

                        this.outputTextArea.append(logMe.getMessage() + "\n");
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: refresh error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: refresh error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>init</code>
     *
     */
    public void init() {

	this.context.setAuthentication(ServiceContext.XML_ENCRYPTION);
	this.context.setAuthorization(new NoAuthorization());
	
	(this.context.getInterface(SecurityPanel.QNAME.toString())).init();
	
        refreshLog();
        initLogNotify();
        refreshSize();
        refreshAts();
        refreshCon();
    }

    private void initLogNotify() {
	
        Hashtable props = new Hashtable();
        props.put(Constants.GSI_SEC_MSG,
		  Constants.SIGNATURE);
	props.put(Constants.AUTHORIZATION, 
		  NoAuthorization.getInstance());

        try {
            manager = NotificationSinkManager.getInstance("Secure");
	    manager.init(props);
            manager.startListening(NotificationSinkManager.CHILD_THREAD);
	    
            sink =
                manager.addListener(
                    OgsiLogging.LOG_MESSAGES, null,
                    new HandleType(this.handle.toString()),
                    (NotificationSinkCallback) this
                );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    * implementation for the NotificationSinkCallback
    */
    public void deliverNotification(ExtensibilityType message)
        throws RemoteException {
        try {
            ServiceDataValuesType serviceDataValues =
                AnyHelper.getAsServiceDataValues(message);

            if (
                (serviceDataValues == null) || serviceDataValues.equals(null) ||
                    (serviceDataValues.get_any() == null)
            ) {
                this.outputTextArea.setText(null);
                this.outputTextArea.append(" no messages logged \n");

                return;
            }

            Object[] logm =
                AnyHelper.getAsObject(
                    serviceDataValues, LogMessageElement.class
                );

            if (logm == null) {
                this.outputTextArea.setText(null);
                this.outputTextArea.append(" no messages logged \n");

                return;
            }

            // clear the log view
            this.outputTextArea.setText(null);

            //repopulate
            for (int i = 0; i < logm.length; i++) {
                try {
                    LogMessageElement logMe = (LogMessageElement) logm[i];
                    String lev = logMe.getLevel().trim();

                    this.outputTextArea.append(logMe.getMessage() + "\n");
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        } catch (Exception exception) {
            exception.printStackTrace();
            throw new RemoteException(exception.getMessage());
        }
    }

    public Object getProperty(String name) {
        return properties.get(name);
    }

    public Object getPersistentProperty(String name) {
        return null;
    }

    public void setPersistentProperty(
        String name,
        Object property
    ) {
    }

    public void flush() {
    }

    public void setProperty(
        String name,
        Object property
    ) {
        properties.put(name, property);
    }

    /**
     * <code>dispose</code>
     *
     */
    public void dispose() {
        try {
            if (this.manager.isListening()) {
                this.manager.removeListener(this.sink);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     *
     * <code>refreshPersistent combo</code>
     *
     * Get the PersistentAttributes service data element.
     *
     */
    private void refreshSize() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

	    OgsiLoggingPortType loggingService =
                locator.getOgsiLoggingPort(this.defaultEndpoint);

	    this.context.setAuthentication((Stub)loggingService);

            // Get the transient attributes service data element
            ExtensibilityType queryResult =
                loggingService.findServiceData(
                    QueryHelper.getNamesQuery(OgsiLogging.LOG_BUFFER_SIZE)
                );

            if (
                (queryResult.get_any() != null) &&
                    (queryResult.get_any().length > 0)
            ) {
                ServiceDataValuesType serviceDataValues =
                    (ServiceDataValuesType) AnyHelper.getAsSingleObject(
                        queryResult, ServiceDataValuesType.class
                    );

                if (serviceDataValues == null) {
                    return;
                }

                Integer size =
                    (Integer) AnyHelper.getAsSingleObject(serviceDataValues);

                // Refresh the size field
                sizeTF.setText(size.toString());
            }
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Logging: refresh Con error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh con: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Logging: refresh con error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }
}
