/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.impl.core.management.OgsiManagement;
import org.globus.ogsa.ServiceData;
import org.globus.ogsa.core.management.NumberServicesElementType;
import org.globus.ogsa.core.management.OgsiManagementPortType;
import org.globus.ogsa.core.management.service.OgsiManagementServiceLocator;
import org.globus.ogsa.types.properties.PropertiesDetailType;
import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.QueryHelper;

import org.gridforum.ogsi.EntryType;
import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.FaultType;
import org.gridforum.ogsi.GridService;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.OGSIServiceGridLocator;
import org.gridforum.ogsi.ServiceDataValuesType;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.Timer;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.net.URL;
import javax.xml.rpc.Stub;

public class OgsiManagementPortTypePanel extends AbstractPortTypePanel {
    private OgsiManagementServiceLocator locator;
    private NumberServicesElementType servicesCount;
    private JComboBox handleCB;
    private JTextField deployedTF;
    private JTextField availableTF;
    private JTextField activeTF;
    private JTextField deactiveTF;
    private Timer timer;

    /**
     * <code>OgsiManagementPortTypePanel</code>
     *
     * Panel to display the OgsiManagementPortType with the service browser.
     *
     */
    public OgsiManagementPortTypePanel() {
        super(" OGSI Management portType ");
        this.locator = new OgsiManagementServiceLocator();

        // Set up required panels.  The main panel with use a BoxLayout
        // manager - the rest will use a FlowLayout.
        JPanel servicePanel = new JPanel();
        servicePanel.setLayout(new BoxLayout(servicePanel, BoxLayout.Y_AXIS));

        JPanel handlePanel = new JPanel();
        handlePanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel serviceDataPanel = new JPanel();
        serviceDataPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel refreshPanel = new JPanel();
        refreshPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        // Add the handle combo box to a panel
        handlePanel.add(new JLabel("Handle: "));
        this.handleCB = new JComboBox();
        handleCB.setEditable(true);
        handlePanel.add(this.handleCB);

        // Add the buttons to another panel
        JButton endServiceBT = new JButton("endService");
        JButton endServiceForceBT = new JButton("endServiceForce");
        JButton activateServiceBT = new JButton("activateService");
        JButton deactivateServiceBT = new JButton("deactivateService");

        buttonPanel.add(endServiceBT);
        buttonPanel.add(endServiceForceBT);
        buttonPanel.add(activateServiceBT);
        buttonPanel.add(deactivateServiceBT);

        endServiceBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    endService();
                    refresh();
                }
            }
        );

        endServiceForceBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    endServiceForce();
                    refresh();
                }
            }
        );

        activateServiceBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    activateService();
                    refresh();
                }
            }
        );

        deactivateServiceBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    deactivateService();
                    refresh();
                }
            }
        );

        // Set up service data labels and text areas
        JLabel deployedLabel = new JLabel("Deployed Services: ");
        deployedTF = new JTextField(4);
        deployedTF.setEditable(false);

        JLabel availableLabel = new JLabel("Available Services: ");
        availableTF = new JTextField(4);
        availableTF.setEditable(false);

        JLabel activeLabel = new JLabel("Active Services: ");
        activeTF = new JTextField(4);
        activeTF.setEditable(false);

        JLabel deactiveLabel = new JLabel("Inactive Services: ");
        deactiveTF = new JTextField(4);
        deactiveTF.setEditable(false);

        serviceDataPanel.add(deployedLabel);
        serviceDataPanel.add(deployedTF);
        serviceDataPanel.add(availableLabel);
        serviceDataPanel.add(availableTF);
        serviceDataPanel.add(activeLabel);
        serviceDataPanel.add(activeTF);
        serviceDataPanel.add(deactiveLabel);
        serviceDataPanel.add(deactiveTF);

        // Setup refresh panel 
        JCheckBox autoUpdateCB = new JCheckBox("Auto update");
        autoUpdateCB.setSelected(false);

        this.timer =
            new Timer(
                5000,
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        refresh();
                    }
                }
            );

        autoUpdateCB.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (timer.isRunning()) {
                        timer.stop();
                    } else {
                        refresh();
                        timer.start();
                    }
                }
            }
        );

        JButton updateBT = new JButton("Refresh");

        updateBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    refresh();
                }
            }
        );

        refreshPanel.add(autoUpdateCB);
        refreshPanel.add(updateBT);

        // Add the handle, button, serviceData and refresh panels to 
        // the servicePanel that is using the BoxLayout Mgr
        servicePanel.add(handlePanel);
        servicePanel.add(buttonPanel);
        servicePanel.add(serviceDataPanel);
        servicePanel.add(refreshPanel);

        // Add the completed Service Panel 
        setLayout(new BorderLayout());
        add(servicePanel, BorderLayout.NORTH);

    }

    /**
     * <code>endService</code>
     *
     * Process the endService button.
     *
     */
    private void endService() {
        try {
            //System.out.println("endService()");
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            // Create a HandleType from the input text field and call endService.
            OgsiManagementPortType mgmtPT =
                locator.getOgsiManagementPort(this.defaultEndpoint);
            HandleType handle =
                new HandleType((String) this.handleCB.getSelectedItem());
	    this.context.setAuthentication((Stub)mgmtPT);
            mgmtPT.endService(handle);
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to end service: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Management: endService error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to end service: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Management: endService error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>endServiceForce</code>
     *
     * Process the endServiceForce button.
     *
     */
    private void endServiceForce() {
        try {
            //System.out.println("endServiceForce()");
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            // Create a HandleType from the input text field and call endServiceForce.
            OgsiManagementPortType mgmtPT =
                locator.getOgsiManagementPort(this.defaultEndpoint);
            HandleType handle =
                new HandleType((String) this.handleCB.getSelectedItem());
	    this.context.setAuthentication((Stub)mgmtPT);
            mgmtPT.endServiceForce(handle);
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to end service force: " + ft.getClass().getName() +
                "\n" + ft.getDescription()[0],
                "OGSI Management: endServiceForce error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to end service force: " + e.getClass().getName() +
                "\n" + e.getMessage(), "OGSI Management: endServiceForce error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>activateService</code>
     *
     * Process the activateService button.
     *
     */
    private void activateService() {
        try {
            //System.out.println("activateService()");
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            // Create a HandleType from the input text field and call activateService.
            OgsiManagementPortType mgmtPT =
                locator.getOgsiManagementPort(this.defaultEndpoint);
            HandleType handle =
                new HandleType((String) this.handleCB.getSelectedItem());
	    this.context.setAuthentication((Stub)mgmtPT);
            mgmtPT.activateService(handle);
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to activate service: " + ft.getClass().getName() +
                "\n" + ft.getDescription()[0],
                "OGSI Management: activateService error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to activate service: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Management: activateService error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>dactivateService</code>
     *
     * Process the deactiveService button.
     *
     */
    private void deactivateService() {
        try {
            //System.out.println("deactivateService()");
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            // Create a HandleType from the input text field and call deactivateService.
            OgsiManagementPortType mgmtPT =
                locator.getOgsiManagementPort(this.defaultEndpoint);
            HandleType handle =
                new HandleType((String) this.handleCB.getSelectedItem());
	    this.context.setAuthentication((Stub)mgmtPT);
            mgmtPT.deactivateService(handle);
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to deactivate service: " + ft.getClass().getName() +
                "\n" + ft.getDescription()[0],
                "OGSI Management: deactivateService error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to deactivate service: " + e.getClass().getName() +
                "\n" + e.getMessage(),
                "OGSI Management: deactivateService error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     *
     * <code>getServiceCounts</code>
     *
     * Get the service counts service data element.
     *
     */
    private void refresh() {
        try {
            if ((this.context == null) || (this.defaultEndpoint == null)) {
                throw new Exception("defaultEndpoint is null");
            }

            OGSIServiceGridLocator managementLocator =
                new OGSIServiceGridLocator();

            GridService mgmtService =
                managementLocator.getGridServicePort(this.defaultEndpoint);
            // Get the service counts service data element
            ExtensibilityType queryResult =
                mgmtService.findServiceData(
                    QueryHelper.getNamesQuery(OgsiManagement.NUMBER_SERVICES)
                );
	    
            if (
                (queryResult.get_any() != null) &&
                    (queryResult.get_any().length > 0)
            ) {
                ServiceDataValuesType serviceDataValues =
                    (ServiceDataValuesType) AnyHelper.getAsSingleObject(
                        queryResult, ServiceDataValuesType.class
                    );
                this.servicesCount =
                    (NumberServicesElementType) AnyHelper.getAsSingleObject(
                        serviceDataValues, NumberServicesElementType.class
                    );

                int nAvailable =
                    this.servicesCount.getNumberAvailableServices();
                int nDeployed = this.servicesCount.getNumberDeployedServices();
                int nActive = this.servicesCount.getNumberActiveServices();
                int nDeactive = this.servicesCount.getNumberDeactiveServices();

                this.availableTF.setText(
                    Integer.toString(
                        this.servicesCount.getNumberAvailableServices()
                    )
                );
                this.deployedTF.setText(
                    Integer.toString(
                        this.servicesCount.getNumberDeployedServices()
                    )
                );
                this.activeTF.setText(
                    Integer.toString(
                        this.servicesCount.getNumberActiveServices()
                    )
                );
                this.deactiveTF.setText(
                    Integer.toString(
                        this.servicesCount.getNumberDeactiveServices()
                    )
                );
            }

            // We know that the service implementing the OgsiManagement
            // portType also implements the service group portType. This
            // will be used to populate the combo box used for the handle
            // in the operations offered by this service.
            queryResult =
                mgmtService.findServiceData(
                    QueryHelper.getNamesQuery(ServiceData.ENTRY)
                );
            if (
                (queryResult.get_any() != null) &&
                    (queryResult.get_any().length > 0)
            ) {
                ServiceDataValuesType serviceDataValues =
                    (ServiceDataValuesType) AnyHelper.getAsSingleObject(
                        queryResult, ServiceDataValuesType.class
                    );

                Object[] entries =
                    AnyHelper.getAsObject(serviceDataValues, EntryType.class);

                // Refresh the list in the combox box.
                this.handleCB.removeAllItems();

                for (int i = 0; i < entries.length; i++) {
                    try {
                        EntryType entry = (EntryType) entries[i];
                        PropertiesDetailType item =
                            (PropertiesDetailType) AnyHelper.getAsSingleObject(
                                entry.getContent(), PropertiesDetailType.class
                            );
                        this.handleCB.addItem(item.getHandle());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (FaultType ft) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh: " + ft.getClass().getName() + "\n" +
                ft.getDescription()[0], "OGSI Management: refresh error",
                JOptionPane.ERROR_MESSAGE
            );
            ft.printStackTrace();

            return;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this,
                "Failed to refresh: " + e.getClass().getName() + "\n" +
                e.getMessage(), "OGSI Management: refresh error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    /**
     * <code>init</code>
     *
     */
    public void init() {
        refresh();
    }

    /**
     * <code>dispose</code>
     *
     */
    public void dispose() {
        // cleanup the time if it is running
        if (this.timer.isRunning()) {
            this.timer.stop();
        }
    }
}
