/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.impl.core.handle.HandleHelper;
import org.globus.ogsa.types.properties.PropertiesDetailType;
import org.globus.ogsa.utils.AnyHelper;

import org.gridforum.ogsi.EntryType;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.LocatorType;

import javax.swing.table.AbstractTableModel;

import java.util.ArrayList;
import java.util.List;


public class RegistryListTableModel extends AbstractTableModel {
    private String[] columns = { "Name", "Handle", "State" };
    private List list;

    public RegistryListTableModel() {
        super();
    }

    public void setList(Object[] entries) {
        this.list = new ArrayList();

        for (int i = 0; i < entries.length; i++) {
            try {
                EntryType entry = (EntryType) entries[i];
                if (AnyHelper.contains(entry.getContent(), PropertiesDetailType.class)) {
                    this.list.add(
                        AnyHelper.getAsSingleObject(
                            entry.getContent(), PropertiesDetailType.class
                        )
                    );
                } else {
                    this.list.add(
                            entry.getMemberServiceLocator().getHandle()[0]
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        fireTableDataChanged();
    }

    public List getList() {
        return this.list;
    }

    public String getLocation(int row) {
        return (String) getValueAt(row, 1);
    }

    public boolean isCellEditable(
        int row,
        int col
    ) {
        return false;
    }

    public String getColumnName(int index) {
        return this.columns[index];
    }

    public int getRowCount() {
        if (this.list == null) {
            return 0;
        }

        return this.list.size();
    }

    public int getColumnCount() {
        return this.columns.length;
    }

    public Object getValueAt(
        int row,
        int column
    ) {
        Object obj = this.list.get(row);
        if (obj instanceof PropertiesDetailType) {
            PropertiesDetailType entry = (PropertiesDetailType) obj; 
            switch (column) {
              case 0:
                  return entry.getName();

              case 1:
                  return entry.getHandle();

              case 2:
                  return "" + entry.getState();
            }
        } else if (obj instanceof HandleType) {
            switch (column) {
                case 0:
                    try {
                        return HandleHelper.getInstanceID((HandleType) obj); 
                    } catch (Exception e) {
                        return "";
                    }
                case 1:
                    return obj.toString();
            }
        }
        return "";
    }
}
