/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.xml.namespace.QName;

import org.globus.ogsa.impl.security.authorization.Authorization;
import org.globus.ogsa.impl.security.authorization.NoAuthorization;
import org.globus.ogsa.impl.security.authorization.SelfAuthorization;
import org.globus.ogsa.impl.security.authorization.HostAuthorization;
import org.globus.ogsa.impl.security.authorization.IdentityAuthorization;

public class SecurityPanel 
    extends AbstractPortTypePanel {

    public static final QName QNAME = 
	new QName("Security", "http://www.globus.org");
    
    private JRadioButton noneAutheRB;
    private JRadioButton gsiXmlSigRB;
    private JRadioButton gsiSecConvRB;
    
    private JRadioButton noneAuthoRB;
    private JRadioButton hostAuthoRB;
    private JRadioButton selfAuthoRB;
    private JRadioButton identityAuthoRB;
    private JTextField identityTF;

    private JComboBox protCB;
    private JComboBox delegCB;

    public SecurityPanel() {
        super(" Message Security ");
	
	JPanel authePanel = new JPanel();
	authePanel.setBorder(BorderFactory.createTitledBorder(" Authentication "));
	
	JPanel authoPanel = new JPanel();
	authoPanel.setBorder(BorderFactory.createTitledBorder(" Authorization "));

        ButtonGroup buttonGroup1 = new ButtonGroup();
	noneAutheRB = new JRadioButton("None");
	gsiXmlSigRB = new JRadioButton("GSI Secure Message");
	gsiSecConvRB = new JRadioButton("GSI Secure Conversation");

        buttonGroup1.add(noneAutheRB);
        buttonGroup1.add(gsiXmlSigRB);
        buttonGroup1.add(gsiSecConvRB);

	protCB = 
	    new JComboBox(new String[] {"Integrity", "Privacy"});

	delegCB = 
	    new JComboBox(new String[] {"None", "Limited", "Full"});

	ActionListener protListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if (context.getAuthentication() ==
				ServiceContext.GSI_XML_SIGNATURE) {
			protCB.setSelectedIndex(0);
		    } else{
			setGSISecConvProtection(protCB.getSelectedIndex());
		    }
		}
	    };

	ActionListener delegListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    setGSISecConvDelegation(delegCB.getSelectedIndex());
		    gsiSecConvRB.setSelected(true);
		}
	    };
	
	protCB.addActionListener(protListener);
	delegCB.addActionListener(delegListener);

	ActionListener autheListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Object source = e.getSource();
		    if (source == noneAutheRB) {
			context.setAuthentication(
                                    ServiceContext.NO_AUTHENTICATION);
		    } else if (source == gsiXmlSigRB) {
			context.setAuthentication(
                                    ServiceContext.GSI_XML_SIGNATURE);
			protCB.setSelectedIndex(0);
		    } else if (source == gsiSecConvRB) {
			setGSISecConvProtection(protCB.getSelectedIndex());
			setGSISecConvDelegation(delegCB.getSelectedIndex());
		    }
		}
	    };
	
	noneAutheRB.addActionListener(autheListener);
	gsiXmlSigRB.addActionListener(autheListener);
	gsiSecConvRB.addActionListener(autheListener);

	authePanel.add(noneAutheRB);
	authePanel.add(gsiXmlSigRB);
	authePanel.add(gsiSecConvRB);
	authePanel.add(new JLabel("Protection:"));
	authePanel.add(protCB);
	authePanel.add(new JLabel("Delegation:"));
	authePanel.add(delegCB);

	ButtonGroup buttonGroup2 = new ButtonGroup();
	noneAuthoRB = new JRadioButton("None");
	hostAuthoRB = new JRadioButton("Host");
	selfAuthoRB = new JRadioButton("Self");
	identityAuthoRB = new JRadioButton("Identity");
	
	identityTF = new JTextField(20);

        buttonGroup2.add(noneAuthoRB);
        buttonGroup2.add(hostAuthoRB);
        buttonGroup2.add(selfAuthoRB);
	buttonGroup2.add(identityAuthoRB);

	authoPanel.add(noneAuthoRB);
	authoPanel.add(hostAuthoRB);
	authoPanel.add(selfAuthoRB);
	authoPanel.add(identityAuthoRB);
	authoPanel.add(identityTF);

	ActionListener authoListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Object source = e.getSource();
		    if (source == noneAuthoRB) {
			context.setAuthorization(
				    NoAuthorization.getInstance()
			);
		    } else if (source == hostAuthoRB) {
			context.setAuthorization(
				    HostAuthorization.getInstance()
			);
		    } else if (source == selfAuthoRB) {
			context.setAuthorization(
				    SelfAuthorization.getInstance()
			);
		    } else if (source == identityAuthoRB) {
			String ident = identityTF.getText();
			IdentityAuthorization auth
			    = new IdentityAuthorization(ident);
			context.setAuthorization(auth);
		    }
		}
	    };

	noneAuthoRB.addActionListener(authoListener);
	hostAuthoRB.addActionListener(authoListener);
	selfAuthoRB.addActionListener(authoListener);
	identityAuthoRB.addActionListener(authoListener);
	
	noneAutheRB.setSelected(true);
	hostAuthoRB.setSelected(true);

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	add(authePanel);
	add(authoPanel);
    }

    protected void setGSISecConvProtection(int prot) {
	switch(prot) {
	case 0:
	    context.setAuthentication(ServiceContext.XML_SIGNATURE);
	    break;
	case 1:
	    context.setAuthentication(ServiceContext.XML_ENCRYPTION);
	}
    }

    protected void setGSISecConvDelegation(int deleg) {
	switch(deleg) {
	case 0:
	    context.setDelegation(ServiceContext.NO_DELEGATION);
	    break;
	case 1:
	    context.setDelegation(ServiceContext.LIMITED_DELEGATION);
	    break;
	case 2:
	    context.setDelegation(ServiceContext.FULL_DELEGATION);
	    break;
	}
    }

    public void init() {
	setAuthentication(this.context.getAuthentication());
	setAuthorization(this.context.getAuthorization());
	setDelegation(this.context.getDelegation());
    }
    
    public void dispose() {
    }
    
    protected void setAuthentication(int authentication) {
	switch(authentication) {
	case ServiceContext.XML_SIGNATURE:
	    protCB.setSelectedIndex(0);
	    break;
	case ServiceContext.XML_ENCRYPTION:
	    protCB.setSelectedIndex(1);
	    gsiSecConvRB.setSelected(true);
	    break;
	case ServiceContext.NO_AUTHENTICATION:
	    noneAutheRB.setSelected(true);
	    break;
	case ServiceContext.GSI_XML_SIGNATURE:
	    gsiXmlSigRB.setSelected(true);
	    break;
	}
    }
    
    protected void setDelegation(int delegation) {
	switch(delegation) {
	case ServiceContext.NO_DELEGATION:
	    delegCB.setSelectedIndex(0);
	    break;
	case ServiceContext.LIMITED_DELEGATION:
	    delegCB.setSelectedIndex(1);
	    break;
	case ServiceContext.FULL_DELEGATION:
	    protCB.setSelectedIndex(2);
	    break;
	}
    }

    protected void setAuthorization(Authorization auth) {
	if (auth instanceof NoAuthorization) {
	    noneAuthoRB.setSelected(true);
	} else if (auth instanceof SelfAuthorization) {
	    selfAuthoRB.setSelected(true);
	} else if (auth instanceof HostAuthorization) {
	    hostAuthoRB.setSelected(true);
	} else if (auth instanceof IdentityAuthorization) {
	    String identity = ((IdentityAuthorization)auth).getIdentity();
	    identityAuthoRB.setSelected(true);
	    identityTF.setText(identity);
	}
    }
}
