/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.client.ServiceURL;
import org.globus.ogsa.utils.MessageUtils;

import org.globus.axis.util.Util;
import org.globus.net.GlobusURLStreamHandlerFactory;

import org.apache.axis.utils.XMLUtils;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import javax.swing.JFrame;

import javax.xml.namespace.QName;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.io.InputStream;

import java.net.URL;

import java.util.Hashtable;
import java.util.Map;


public class ServiceBrowserFrame extends JFrame {

    public static final String DEFAULT_SERVICE_NAME =
                                  "core/registry/ContainerRegistryService";

    private static final String PANEL_CONFIG_FILE = "client-gui-config.xml";
    private ServiceBrowserImpl browser;
    private boolean isRoot = false;

    public ServiceBrowserFrame(
        URL handle,
        Map mapper,
        boolean isRoot
    ) {
        this(handle, mapper);
        this.isRoot = true;
    }

    public boolean isRoot() {
        return this.isRoot;
    }

    public ServiceBrowserFrame(
        URL handle,
        Map mapper
    ) {
        if (handle == null) {
            setTitle("OGSA Service Browser");
        } else {
            setTitle(
                "[" + handle.toExternalForm() + "] - OGSA Service Browser"
            );
        }

        this.browser = new ServiceBrowserImpl(this);
        this.browser.setHandle(handle);
        this.browser.init(mapper);

        getContentPane().add(this.browser);
        pack();

        WindowListener l =
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    browser.dispose();
                }
            };

        addWindowListener(l);
    }

    public static void main(String[] args) {
        URL url = null;

        URL.setURLStreamHandlerFactory(new GlobusURLStreamHandlerFactory());
        Util.registerTransport();

        String contactString = null;
        if (args.length > 0) {
            contactString = args[0];
        }

        try {
            url = ServiceURL.getURL(contactString, DEFAULT_SERVICE_NAME);
        } catch (Exception e) {
            System.err.println("Invalid URL: " + e.getMessage());
            System.exit(1);
        }

        Hashtable portTypePanels = new Hashtable();
        Hashtable serviceDataPanels = new Hashtable();
        Hashtable panels = new Hashtable();

        try {
            loadPanels(portTypePanels, "portType");
            loadPanels(serviceDataPanels, "serviceData");
        } catch (PanelConfigException pce) {
            pce.printStackTrace();

            return;
        }

        panels.put("portType", portTypePanels);
        panels.put("serviceData", serviceDataPanels);

        ServiceBrowserFrame browser =
            new ServiceBrowserFrame(url, panels, true);

        WindowListener l =
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
            };

        browser.addWindowListener(l);
        browser.setVisible(true);
    }

    private static void loadPanels(
        Hashtable panels,
        String panelType
    ) throws PanelConfigException {
        InputStream input = null;

        try {
            ClassLoader loader = ServiceBrowserFrame.class.getClassLoader();
            input = loader.getResourceAsStream(PANEL_CONFIG_FILE);

            if (input == null) {
                throw new PanelConfigException(
                    "Could not find: " + PANEL_CONFIG_FILE
                );
            }

            QName panelQName = null;
            Document doc = XMLUtils.newDocument(input);
            NodeList nodes = doc.getElementsByTagName("panel");

            for (int i = 0; i < nodes.getLength(); i++) {
                Element element = (Element) nodes.item(i);
                String panelKey = element.getAttribute(panelType);

                if (panelKey != null) {
                    panelQName = XMLUtils.getQNameFromString(panelKey, element);

                    String id = null;

                    if (panelQName.getNamespaceURI().length() == 0) {
                        id = panelQName.getLocalPart();
                    } else {
                        id = panelQName.getNamespaceURI() + ":" +
                            panelQName.getLocalPart();
                    }

                    panels.put(
                        id, Class.forName(element.getAttribute("class"))
                    );
                }
            }
        } catch (Exception e) {
            throw new PanelConfigException(MessageUtils.toString(e));
        } finally {
            if (input != null) {
                try {
                    input.close();
                } catch (Exception e) {
                    throw new PanelConfigException(MessageUtils.toString(e));
                }
            }
        }
    }
}
