/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.GSIUtils;
import org.globus.ogsa.utils.MessageUtils;
import org.globus.ogsa.utils.QueryHelper;
import org.globus.ogsa.utils.XmlFactory;
import org.globus.ogsa.wsdl.GSR;
import org.globus.ogsa.wsdl.extensions.PortTypeExtensibilityElement;

import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.GridService;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.OGSIServiceGridLocator;
import org.gridforum.ogsi.ServiceDataType;
import org.gridforum.ogsi.TargetInvalidFaultType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.w3c.dom.Element;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JSplitPane;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.net.URL;
import java.net.URLClassLoader;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;


public class ServiceBrowserImpl extends JPanel implements ServiceBrowser {
    static Log logger = LogFactory.getLog(ServiceBrowserImpl.class.getName());
    private OGSIServiceGridLocator gridServiceLocator =
        new OGSIServiceGridLocator();
    private URL handle;
    private GSR gsr = null;
    private ServiceContext context;
    private ServiceBrowserFrame parentWindow;
    private XMLTree wsdlTree;
    private JTextField addressTF;
    private JButton goBT;
    private JButton backBT;
    private JButton forwardBT;
    private JButton newWindowBT;
    private JButton closeBT;
    private JButton refreshBT;
    private JPanel interfacePanel;
    private static PanelClassLoader panelClassLoader = null;
    private static Hashtable panelUrls = new Hashtable();
    private HistoryList history = new HistoryList();
    private int historyIndex = -1;
    private ServiceDataDescriptionListTableModel serviceDataModel;
    private Collection portTypes;
    private JCheckBox showDynamicCB;
    private XMLTree serviceDataTree;
    private Hashtable loadedPanels = new Hashtable();

    public ServiceBrowserImpl(ServiceBrowserFrame parentWindow) {
        this.parentWindow = parentWindow;

        setLayout(new BorderLayout());

        this.interfacePanel = new JPanel();
        this.interfacePanel.setLayout(
            new BoxLayout(this.interfacePanel, BoxLayout.Y_AXIS)
        );

        JPanel addressPanel = new JPanel();
        JPanel buttonPanel = new JPanel();
        JPanel headPanel = new JPanel();

        addressPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        headPanel.setLayout(new BoxLayout(headPanel, BoxLayout.Y_AXIS));

        this.addressTF = new JTextField(55);
        this.goBT = new JButton("Go");
        this.backBT = new JButton("Back");
        this.forwardBT = new JButton("Forward");
        this.newWindowBT = new JButton("New Window");
        this.closeBT = new JButton("Close");
        this.refreshBT = new JButton("Refresh");
        this.showDynamicCB = new JCheckBox("Show dynamic gui");
        this.showDynamicCB.setSelected(false);

        addressPanel.add(this.addressTF);
        addressPanel.add(this.goBT);
        buttonPanel.add(this.backBT);
        buttonPanel.add(this.forwardBT);
        buttonPanel.add(this.newWindowBT);
        buttonPanel.add(this.closeBT);
        buttonPanel.add(this.refreshBT);
        buttonPanel.add(this.showDynamicCB);

        this.goBT.addActionListener(new GoAction());
        this.newWindowBT.addActionListener(new NewWindowAction());
        this.closeBT.addActionListener(new CloseAction());
        this.refreshBT.addActionListener(new RefreshAction());
        this.backBT.addActionListener(new BackAction());
        this.forwardBT.addActionListener(new ForwardAction());

        headPanel.add(buttonPanel);
        headPanel.add(addressPanel);
        add(headPanel, BorderLayout.NORTH);

        try {
            this.wsdlTree = new XMLTree(XmlFactory.newDocument());
            this.serviceDataTree = new XMLTree(XmlFactory.newDocument());
        } catch (Exception e) {
            e.printStackTrace();
        }

        JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.add("Services", new JScrollPane(this.interfacePanel));

        JScrollPane wsdlPanel = new JScrollPane(this.wsdlTree);
        wsdlPanel.setPreferredSize(new Dimension(200, 200));
        tabbedPane.add("WSDL", wsdlPanel);

        this.serviceDataModel = new ServiceDataDescriptionListTableModel();

        final JTable table = new JTable();
        table.setModel(this.serviceDataModel);

        JScrollPane serviceDataTablePane = new JScrollPane(table);
        serviceDataTablePane.setPreferredSize(new Dimension(200, 200));

        JScrollPane serviceDataValuePane =
            new JScrollPane(this.serviceDataTree);
        serviceDataValuePane.setPreferredSize(new Dimension(200, 100));
        serviceDataValuePane.setBorder(
            BorderFactory.createTitledBorder(" Values ")
        );

	JSplitPane serviceDataPanel = 
	    new JSplitPane(JSplitPane.VERTICAL_SPLIT,
			   false,
			   serviceDataTablePane,
			   serviceDataValuePane);

        tabbedPane.add("Service Data", serviceDataPanel);

        add(tabbedPane, BorderLayout.CENTER);

        table.addMouseListener(
            new MouseAdapter() {
                public void mousePressed(MouseEvent e) {
                    if (e.getClickCount() < 1) {
                        return;
                    }

                    int i = table.getSelectedRow();
                    QName qname = serviceDataModel.getName(i);
                    GridService service = null;
                    ExtensibilityType queryResult = null;

                    try {
                        service =
                            gridServiceLocator.getGridServicePort(
                                new HandleType(handle.toString())
                            );

			GSIUtils.setDefaultGSIProperties((Stub)service);
			context.setAuthentication((Stub)service);

                        queryResult =
                            service.findServiceData(
                                QueryHelper.getNamesQuery(qname)
                            );
                    } catch (TargetInvalidFaultType fault) {
                        try {
                            queryResult =
                                service.findServiceData(
                                    QueryHelper.getNamesQuery(
                                        new QName("", qname.getLocalPart())
                                    )
                                );
                        } catch (Exception exc) {
                            exc.printStackTrace();

                            return;
                        }
                    } catch (Exception exc) {
                        exc.printStackTrace();

                        return;
                    }

                    try {
                        Element result =
                            AnyHelper.getAsSingleElement(queryResult);
                        serviceDataTree.setModel(new XMLTreeModel(result));
                    } catch (Exception exc) {
                        exc.printStackTrace();
                    }
                }
            }
        );
    }

    public void setHandle(URL handle) {
        this.handle = handle;
    }

    public void init(Map mapper) {
        this.loadedPanels.clear();
        if (this.handle == null) {
            return;
        }

        this.context = new ServiceContext();
        this.context.setBrowser(this);
        this.context.setPanelMapper(mapper);

        this.addressTF.setText(this.handle.toExternalForm());

        if (historyIndex == -1) {
            this.history.add(this.addressTF.getText());
            this.historyIndex++;
        }

        Collection ports = null;

        try {
            this.gsr = GSR.newInstance(new HandleType(this.handle.toString()));
            this.wsdlTree.setModel(new XMLTreeModel(this.gsr.getDocument()));
            this.portTypes = this.gsr.getGWSDLPortTypes();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to obtain WSDL: " + 
		MessageUtils.getErrorMessage(e), "Error",
                JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }

	addPanel(new SecurityPanel(),  SecurityPanel.QNAME);

        for (Iterator iterator = this.portTypes.iterator(); iterator.hasNext();) {
            QName name = null;
            JComponent portTypePanel = null;

            try {
                PortTypeExtensibilityElement element =
                    (PortTypeExtensibilityElement) iterator.next();
                ServiceDataType[] serviceData = element.getServiceData();

                for (
                    int i = 0;
                        (serviceData != null) && (i < serviceData.length);
                        i++
                ) {
                    QName serviceDataName =
                        new QName(
                            element.getName().getNamespaceURI(),
                            serviceData[i].getName().toString()
                        );
                    addPanel(
                        getServiceDataPanel(serviceDataName), serviceDataName
                    );
                }

                this.serviceDataModel.addList(serviceData, element.getName());

                name = element.getName();

                logger.debug(
                    "Qname: " + name.getNamespaceURI() + ":" +
                    name.getLocalPart()
                );

                portTypePanel = getPortTypePanel(name);

                //Fall back to the dynamic port type panel
                if ((portTypePanel == null) && this.showDynamicCB.isSelected()) {
                    portTypePanel =
                        new DynamicPortTypePanel(
                            "Dynamic Panel for " + name.getLocalPart()
                        );
                    ((DynamicPortTypePanel) portTypePanel).setPortType(name);
                }
            } catch (Exception e) {
                JOptionPane.showMessageDialog(
                    this,
                    "Failed to get panel for port type: " + name +
                    e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE
                );
                e.printStackTrace();

                continue;
            }

            addPanel(portTypePanel, name);
        }

        this.serviceDataModel.endList();
    }

    private void addPanel(JComponent panel,
			  QName qname) {
        ServiceInterface gsInterface = (ServiceInterface) panel;

        if (gsInterface == null) {
            return;
        }
        String panelName = panel.getClass().getName();
        if (this.loadedPanels.get(panelName) != null) {
            return;
        }
        this.loadedPanels.put(panelName, panelName);

        gsInterface.setHandle(this.handle);

        try {
            gsInterface.setGSR(this.gsr);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to set gsr for port type: " + e.getMessage(),
                "Error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();
        }

        gsInterface.setContext(this.context);

        // add itself to context
        this.context.addInterface(qname.toString(), gsInterface);

        // init panel - just in case
        gsInterface.init();

        // add to the right panel
        this.interfacePanel.add(panel);
    }

    public JComponent getPortTypePanel(QName portType)
        throws Exception {
        String qname =
            portType.getNamespaceURI() + ":" + portType.getLocalPart();
        logger.debug("Qname: " + qname);

        Map mapper = this.context.getPortTypePanelMapper();
        Object obj = mapper.get(qname);

        if (obj == null) {
            obj = mapper.get(portType.getLocalPart());
        }

        return getComponent(obj);
    }

    public JComponent getServiceDataPanel(QName serviceDataName)
        throws Exception {
        Object obj =
            this.context.getServiceDataPanelMapper().get(
                serviceDataName.getLocalPart()
            );

        return getComponent(obj);
    }

    private JComponent getComponent(Object obj) throws Exception {
        if (obj == null) {
            return null;
        }

        if (!(obj instanceof Class)) {
            throw new Exception("Object is not an instance of class");
        }

        Class clazz = (Class) obj;

        if (
            !JComponent.class.isAssignableFrom(clazz) &&
                !ServiceInterface.class.isAssignableFrom(clazz)
        ) {
            throw new Exception("Class is not of right type");
        }

        return (JComponent) clazz.newInstance();
    }

    public void dispose() {
        // clean up first
        this.context.clear();
        this.interfacePanel.removeAll();
    }

    public void closeService() {
        this.parentWindow.dispose();

        if (this.parentWindow.isRoot()) {
            System.exit(0);
        }
    }

    class GoAction implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            refreshService(addressTF.getText());
        }
    }

    class RefreshAction implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            refreshGSR();
        }
    }

    class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            closeService();
        }
    }

    class NewWindowAction implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            newWindow(addressTF.getText());
        }
    }

    class BackAction implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            if (historyIndex <= 0) {
                return;
            }

            ;
            historyIndex--;

            String historyAddress = (String) history.get(historyIndex);
            addressTF.setText(historyAddress);
            refreshService(addressTF.getText(), true);
        }
    }

    class ForwardAction implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            if (historyIndex >= (history.size() - 1)) {
                return;
            }

            ;
            historyIndex++;

            String historyAddress = (String) history.get(historyIndex);
            addressTF.setText(historyAddress);
            refreshService(addressTF.getText(), true);
        }
    }

    public void newWindow(String location) {
        try {
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR) );
            URL url = new URL(location);
            ServiceBrowserFrame browser =
                new ServiceBrowserFrame(url, this.context.getPanelMapper());
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
            browser.setVisible(true);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to open new window: " + 
		MessageUtils.getErrorMessage(e),
                "ServiceBrowser: new window error", JOptionPane.ERROR_MESSAGE
            );
            System.err.println(MessageUtils.toString(e));

            return;
        }
    }

    public void refreshGSR() {
        try {
            GSR gsr = GSR.newInstance(new HandleType(this.handle.toString()));
            refreshService(addressTF.getText());
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to refresh service: " + 
		MessageUtils.getErrorMessage(e),
                "ServiceBrowser: refresh error", JOptionPane.ERROR_MESSAGE
            );
            System.err.println(MessageUtils.toString(e));
        }
    }

    public void refreshService(String address) {
        refreshService(address, false);
    }

    public void refreshService(
        String address,
        boolean keepHistory
    ) {
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR) );
        URL url = null;

        try {
            url = new URL(address);
        } catch (Exception e) {
            e.printStackTrace();

            return;
        }

        setHandle(url);
        dispose();
        init(this.context.getPanelMapper());
        
        if (url == null) {
                    parentWindow.setTitle("OGSA Service Browser");
                } else {
                    parentWindow.setTitle(
                        "[" + url.toExternalForm() + "] - OGSA Service Browser"
                    );
                }

        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
        repaint();

        if (
            (this.historyIndex >= 0) &&
                ((String) this.history.get(this.historyIndex)).equals(address)
        ) {
            return;
        }

        if (!keepHistory) {
            this.history.removeAfter(historyIndex);
            this.history.add(address);
            historyIndex++;
        }
    }
}


class PanelClassLoader extends URLClassLoader {
    public PanelClassLoader(URL url) {
        super(new URL[] { url }, PanelClassLoader.class.getClassLoader());
    }

    public void addPanelURL(URL url) {
        super.addURL(url);
    }
}


class HistoryList extends ArrayList {
    public void removeAfter(int index) {
        if ((index + 1) < size()) {
            removeRange(index + 1, size());
        }
    }
}
