/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.impl.security.Constants;
import org.globus.ogsa.impl.security.authorization.Authorization;
import org.globus.axis.gsi.GSIConstants;
import org.globus.gsi.proxy.ProxyPolicyHandler;
import org.globus.gsi.proxy.IgnoreProxyPolicyHandler;

import javax.xml.rpc.Stub;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;


public class ServiceContext {

    public static final int NO_AUTHENTICATION = 0;

    // for GSI XML Signature
    public static final int GSI_XML_SIGNATURE = 3;

    // for GSI Secure Conversation
    public static final int XML_ENCRYPTION = 1;
    public static final int XML_SIGNATURE = 2;
    public static final int NO_DELEGATION = 0;
    public static final int LIMITED_DELEGATION = 1;
    public static final int FULL_DELEGATION = 2;

    private Map panelMapper;
    private Map map;
    private ServiceBrowser browser;
    private int authentication = NO_AUTHENTICATION;
    private int delegationType = NO_DELEGATION;
    private Authorization authorization;

    private static ProxyPolicyHandler defaultPolicyHandler =
	new IgnoreProxyPolicyHandler();

    public ServiceContext() {
        this.map = new HashMap();
    }

    public Map getPanelMapper() {
        return this.panelMapper;
    }

    public Map getPortTypePanelMapper() {
        return (Map) this.panelMapper.get("portType");
    }

    public Map getServiceDataPanelMapper() {
        return (Map) this.panelMapper.get("serviceData");
    }

    public ServiceBrowser getBrowser() {
        return this.browser;
    }

    public void setBrowser(ServiceBrowser browser) {
        this.browser = browser;
    }

    public void setPanelMapper(Map mapper) {
        this.panelMapper = mapper;
    }

    public ServiceInterface getInterface(String name) {
        return (ServiceInterface) this.map.get(name.toLowerCase());
    }

    public void addInterface(
        String name,
        ServiceInterface gsInterface
    ) {
        this.map.put(name.toLowerCase(), gsInterface);
    }

    public void clear() {
        Iterator iter = this.map.entrySet().iterator();
        ServiceInterface gsInt;

        while (iter.hasNext()) {
            gsInt = (ServiceInterface) ((Map.Entry) iter.next()).getValue();
            gsInt.dispose();
        }

        this.map.clear();
    }

    public void setAuthentication(Stub stub) {
        if (this.authentication == XML_SIGNATURE) {
            stub._setProperty(Constants.GSI_SEC_CONV, 
			      Constants.SIGNATURE);
	    stub._setProperty(Constants.GSI_SEC_MSG,
			      Constants.NONE);
        } else if (this.authentication == XML_ENCRYPTION) {
            stub._setProperty(Constants.GSI_SEC_CONV, 
			      Constants.ENCRYPTION);
	    stub._setProperty(Constants.GSI_SEC_MSG,
			      Constants.NONE);
        } else if (this.authentication == NO_AUTHENTICATION) {
            stub._setProperty(Constants.GSI_SEC_CONV, 
			      Constants.NONE);
	    stub._setProperty(Constants.GSI_SEC_MSG,
			      Constants.NONE);
        } else if (this.authentication == GSI_XML_SIGNATURE) {
	    stub._setProperty(Constants.GSI_SEC_CONV, 
			      Constants.NONE);
	    stub._setProperty(Constants.GSI_SEC_CONV,
			      Constants.SIGNATURE);
	}

	if (this.authorization != null) {
	    stub._setProperty(Constants.AUTHORIZATION,
			      this.authorization);
	}

        if (this.delegationType == NO_DELEGATION) {
            stub._setProperty(
                GSIConstants.GSI_MODE, GSIConstants.GSI_MODE_NO_DELEG
            );
        } else if (this.delegationType == LIMITED_DELEGATION) {
            stub._setProperty(
                GSIConstants.GSI_MODE, GSIConstants.GSI_MODE_LIMITED_DELEG
            );
        } else if (this.delegationType == FULL_DELEGATION) {
            stub._setProperty(
                GSIConstants.GSI_MODE, GSIConstants.GSI_MODE_FULL_DELEG
            );
        }

	stub._setProperty(Constants.GRIM_POLICY_HANDLER,
			  defaultPolicyHandler);
    }

    public void setAuthentication(int authentication) {
        this.authentication = authentication;
    }

    public int getAuthentication() {
        return this.authentication;
    }

    public void setDelegation(int delegationType) {
        this.delegationType = delegationType;
    }

    public int getDelegation() {
        return this.delegationType;
    }

    public void setAuthorization(Authorization auth) {
        this.authorization = auth;
    }

    public Authorization getAuthorization() {
	return this.authorization;
    }

}
