/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.gridforum.ogsi.ServiceDataType;

import javax.swing.table.AbstractTableModel;

import javax.xml.namespace.QName;

import java.util.ArrayList;
import java.util.List;


public class ServiceDataDescriptionListTableModel extends AbstractTableModel {
    private String[] columns =
    {
        "Name", "PortType", "Type", "Min", "Max", "Mutability", "Modifiable",
        "Nillable"
    };
    private List list;
    private boolean refresh;

    class Entry {
        ServiceDataType serviceData;
        QName portType;

        Entry(
            ServiceDataType serviceData,
            QName portType
        ) {
            this.serviceData = serviceData;
            this.portType = portType;
        }
    }

    public ServiceDataDescriptionListTableModel() {
        super();
    }

    public void endList() {
        this.refresh = true;
    }

    public void addList(
        ServiceDataType[] serviceData,
        QName portType
    ) {
        if (this.refresh) {
            list = null;
            this.refresh = false;
        }

        if (serviceData == null) {
            return;
        }

        if (list == null) {
            list = new ArrayList();
        }

        for (int i = 0; i < serviceData.length; i++) {
            list.add(new Entry(serviceData[i], portType));
        }

        fireTableDataChanged();
    }

    public List getList() {
        return this.list;
    }

    public boolean isCellEditable(
        int row,
        int col
    ) {
        return false;
    }

    public String getColumnName(int index) {
        return this.columns[index];
    }

    public int getRowCount() {
        if (this.list == null) {
            return 0;
        }

        return this.list.size();
    }

    public int getColumnCount() {
        return this.columns.length;
    }

    public Object getValueAt(
        int row,
        int column
    ) {
        Entry entry = (Entry) this.list.get(row);

        switch (column) {
        case 0:
            return entry.serviceData.getName().toString();

        case 1:
            return entry.portType.toString();

        case 2:
            return entry.serviceData.getType().toString();

        case 3:
            return entry.serviceData.getMinOccurs().toString();

        case 4:
            return entry.serviceData.getMaxOccurs();

        case 5:
            return entry.serviceData.getMutability().toString();

        case 6:
            return "" + entry.serviceData.isModifiable();

        case 7:
            return "" + entry.serviceData.isNillable();
        }

        return "N/A";
    }

    public QName getName(int row) {
        Entry entry = (Entry) this.list.get(row);

        return new QName(
            entry.portType.getNamespaceURI(),
            entry.serviceData.getName().toString()
        );
    }
}
