/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.impl.security.Constants;
import org.globus.ogsa.impl.security.authorization.NoAuthorization;
import org.globus.ogsa.impl.security.authorization.SelfAuthorization;
import org.globus.ogsa.NotificationSinkCallback;
import org.globus.ogsa.ServiceData;
import org.globus.ogsa.client.managers.NotificationSinkManager;
import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.GSIUtils;
import org.globus.ogsa.utils.QueryHelper;
import org.globus.ogsa.utils.XmlFactory;

import org.globus.axis.gsi.GSIConstants;

import org.gridforum.ogsi.EntryType;
import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.GridService;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.OGSIServiceGridLocator;
import org.gridforum.ogsi.ServiceDataValuesType;

import org.w3c.dom.Element;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.Timer;

import javax.xml.rpc.Stub;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.rmi.RemoteException;

import java.util.Hashtable;
import java.util.List;


public class ServiceGroupEntryServiceDataPanel extends AbstractPortTypePanel
    implements NotificationSinkCallback {
    private RegistryListTableModel model;
    private Timer timer;
    private XMLTree entryTree;
    private Hashtable properties = new Hashtable();
    private NotificationSinkManager manager;
    private String sink;

    public ServiceGroupEntryServiceDataPanel() {
        super(" Service Group Entry Inspection ");

        JCheckBox autoUpdateCB = new JCheckBox("Auto update");
        autoUpdateCB.setSelected(false);

        this.timer =
            new Timer(
                5000,
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        getRegistry();
                    }
                }
            );

        autoUpdateCB.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (timer.isRunning()) {
                        timer.stop();
                    } else {
                        getRegistry();
                        timer.start();
                    }
                }
            }
        );

        JButton updateBT = new JButton("Refresh");

        updateBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    getRegistry();
                }
            }
        );

        this.model = new RegistryListTableModel();

        final JTable table = new JTable();

        table.setModel(this.model);

        setLayout(new BorderLayout());

        try {
            this.entryTree = new XMLTree(XmlFactory.newDocument());
        } catch (Exception e) {
            e.printStackTrace();
        }

        JPanel btPanel = new JPanel();
        btPanel.add(autoUpdateCB);
        btPanel.add(updateBT);

        JTabbedPane tbPane = new JTabbedPane();

        JScrollPane p1 = new JScrollPane(table);
        p1.setPreferredSize(new Dimension(200, 150));

        JScrollPane p2 = new JScrollPane(this.entryTree);
        p2.setPreferredSize(new Dimension(200, 150));

        tbPane.add("Table", p1);
        tbPane.add("ServiceGroup Entries", p2);

        add(tbPane, BorderLayout.CENTER);
        add(btPanel, BorderLayout.SOUTH);

        table.addMouseListener(
            new MouseAdapter() {
                public void mousePressed(MouseEvent e) {
                    if (e.getClickCount() < 2) {
                        return;
                    }

                    int i = table.getSelectedRow();

                    String location = model.getLocation(i);

                    try {
                        ServiceBrowser browser = context.getBrowser();
                        browser.refreshService(location);
                    } catch (Exception ee) {
                        ee.printStackTrace();
                    }
                }
            }
        );
    }

    private void getRegistry() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        String entryDoc = null;
        OGSIServiceGridLocator registryService = new OGSIServiceGridLocator();

        try {
            GridService registry =
                registryService.getGridServicePort(this.defaultEndpoint);
	    GSIUtils.setDefaultGSIProperties(
	                    (Stub) registry, this.defaultEndpoint
			    );
            ExtensibilityType queryResult =
                registry.findServiceData(
                    QueryHelper.getNamesQuery(ServiceData.ENTRY)
                );
            if (
                (queryResult.get_any() != null) &&
                    (queryResult.get_any().length > 0)
            ) {
                Element element = AnyHelper.getAsSingleElement(queryResult);
                this.entryTree.setModel(new XMLTreeModel(element));

                ServiceDataValuesType serviceDataValues =
                    (ServiceDataValuesType) AnyHelper.getAsSingleObject(
                        queryResult, ServiceDataValuesType.class
                    );

                if (serviceDataValues.get_any() != null) {
                    this.model.setList(
                        AnyHelper.getAsObject(
                            serviceDataValues, EntryType.class
                        )
                    );
                }
            }
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to get registry: " + e.getMessage(),
                "Registry: getRegistry error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    public void init() {
        getRegistry();

        // for notification: sets authorization type
        setProperty(Constants.AUTHORIZATION, SelfAuthorization.getInstance());

        // sets gsi mode
        setProperty(GSIConstants.GSI_MODE, GSIConstants.GSI_MODE_NO_DELEG);

        addRegistryListener();
    }

    public void dispose() {
        if (this.timer.isRunning()) {
            this.timer.stop();
        }

        try {
            if (this.manager.isListening()) {
                this.manager.removeListener(this.sink);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void addRegistryListener() {
        try {
            // TODO: the right way of doing this is to check weather the defaultEndpoint service
            // supports the Source Service and exposes a Entry topic
            this.manager = NotificationSinkManager.getManager();
            this.manager.startListening(NotificationSinkManager.CHILD_THREAD);
            this.sink =
                this.manager.addListener(
                    ServiceData.ENTRY, null,
                    new HandleType(this.handle.toString()), this
                );
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }

    public Object getProperty(String name) {
        return this.properties.get(name);
    }

    public Object getPersistentProperty(String name) {
        return null;
    }

    public void setProperty(
        String name,
        Object property
    ) {
        this.properties.put(name, property);
    }

    public void setPersistentProperty(
        String name,
        Object property
    ) {
    }

    public void flush() {
    }

    public void deliverNotification(ExtensibilityType message)
        throws RemoteException {
        try {
            Element element = AnyHelper.getAsSingleElement(message);
            this.entryTree.setModel(new XMLTreeModel(element));

            ServiceDataValuesType serviceDataValues =
                (ServiceDataValuesType) AnyHelper.getAsSingleObject(
                    message, ServiceDataValuesType.class
                );
            this.model.setList(
                AnyHelper.getAsObject(serviceDataValues, EntryType.class)
            );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
