/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.utils.AnyHelper;
import org.globus.ogsa.utils.ExtendedDateTimeHelper;
import org.globus.ogsa.utils.GSIUtils;
import org.globus.ogsa.utils.ServiceGroupUtils;

import org.gridforum.ogsi.ExtensibilityType;
import org.gridforum.ogsi.HandleType;
import org.gridforum.ogsi.LocatorType;
import org.gridforum.ogsi.OGSIServiceGridLocator;
import org.gridforum.ogsi.ServiceGroupRegistration;
import org.gridforum.ogsi.holders.LocatorTypeHolder;

import org.apache.axis.utils.XMLUtils;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import javax.xml.rpc.Stub;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.ByteArrayInputStream;


public class ServiceGroupRegistrationPortTypePanel extends AbstractPortTypePanel {
    private JTextField unregisterHandleTF;
    private JTextField registerHandleTF;
    private JTextArea xmlTA;
    private OGSIServiceGridLocator registryService =
        new OGSIServiceGridLocator();

    public ServiceGroupRegistrationPortTypePanel() {
        super(" Registry ");

        unregisterHandleTF = new JTextField(25);

        JButton unregisterBT = new JButton("Unregister");

        unregisterBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    unregister();
                }
            }
        );

        JPanel unregisterPanel = new JPanel();
        unregisterPanel.setBorder(BorderFactory.createEtchedBorder());
        unregisterPanel.add(new JLabel("Handle: "));
        unregisterPanel.add(unregisterHandleTF);
        unregisterPanel.add(unregisterBT);

        registerHandleTF = new JTextField(25);

        JButton registerBT = new JButton("Register");

        registerBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    register();
                }
            }
        );

        xmlTA = new JTextArea(7, 5);

        JPanel registerPanel = new JPanel();
        registerPanel.setBorder(BorderFactory.createEtchedBorder());
        registerPanel.setLayout(new BorderLayout());

        JPanel registerXmlPanel = new JPanel();
        registerXmlPanel.setLayout(new BorderLayout());
        registerXmlPanel.add(
            new JLabel("XML description: "), BorderLayout.WEST
        );
        registerXmlPanel.add(new JScrollPane(xmlTA), BorderLayout.CENTER);

        JPanel registerSubPanel = new JPanel();
        registerSubPanel.add(new JLabel("Handle: "));
        registerSubPanel.add(registerHandleTF);
        registerSubPanel.add(registerBT);

        registerPanel.add(registerSubPanel, BorderLayout.NORTH);
        registerPanel.add(registerXmlPanel, BorderLayout.CENTER);

        setLayout(new BorderLayout());
        add(unregisterPanel, BorderLayout.NORTH);
        add(registerPanel, BorderLayout.CENTER);
    }

    private void unregister() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        String handleInput = unregisterHandleTF.getText();

        try {
            ServiceGroupRegistration registry =
                this.registryService.getServiceGroupRegistrationPort(
                    this.defaultEndpoint
                );

            GSIUtils.setDefaultGSIProperties(
                (Stub) registry, this.defaultEndpoint
            );
            registry.remove(
                ServiceGroupUtils.getLocatorEquivalenceExpression(
                    new HandleType(handleInput)
                )
            );

            unregisterHandleTF.setText("");
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to unregister: " +  getErrorMessage(e),
                "Registry: unregister error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }

    private void register() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        String handleInput = registerHandleTF.getText();
        String desc = xmlTA.getText();

        try {
            ServiceGroupRegistration registry =
                this.registryService.getServiceGroupRegistrationPort(
                    this.defaultEndpoint
                );

            GSIUtils.setDefaultGSIProperties(
                (Stub) registry, this.defaultEndpoint
            );

            LocatorType locator = new LocatorType();
            locator.setHandle(new HandleType[] { new HandleType(handleInput) });

            ExtensibilityType extensibility = null;

            if ((desc != null) && (!desc.trim().equals(""))) {
                Document doc =
                    XMLUtils.newDocument(
                        new ByteArrayInputStream(desc.getBytes())
                    );
                Element element = doc.getDocumentElement();
                extensibility = AnyHelper.getExtensibility(element);
            }

            LocatorTypeHolder holder = new LocatorTypeHolder(locator);
            registry.add(holder, 
			 extensibility, 
			 ExtendedDateTimeHelper.INFINITY);

        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to register: " + getErrorMessage(e),
                "Registry: register error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }
    }
}
