/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.samples.notification.SinkListenerPortType;
import org.globus.ogsa.samples.notification.service.SinkListenerServiceLocator;
import org.globus.ogsa.utils.GSIUtils;

import org.gridforum.ogsi.HandleType;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import javax.xml.rpc.Stub;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


public class SinkListenerPortTypePanel extends AbstractPortTypePanel {
    private JTextField notificationSourceTF;
    private JTextField notificationTopicTF;

    public SinkListenerPortTypePanel() {
        super(" Sample Sink Listener ");

        setLayout(new BorderLayout());

        this.notificationSourceTF = new JTextField(25);
        this.notificationTopicTF = new JTextField(25);

        JButton addListenerBT = new JButton("Add");
        JButton removeListenerBT = new JButton("Remove");
        JPanel listenerPanel = new JPanel();
        listenerPanel.setBorder(BorderFactory.createEtchedBorder());
        listenerPanel.add(new JLabel("Source: "));
        listenerPanel.add(this.notificationSourceTF);
        listenerPanel.add(new JLabel("Topic: "));
        listenerPanel.add(this.notificationTopicTF);
        listenerPanel.add(addListenerBT);
        listenerPanel.add(removeListenerBT);
        addListenerBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    addListener();
                }
            }
        );
        removeListenerBT.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeListener();
                }
            }
        );

        add(listenerPanel, BorderLayout.NORTH);
    }

    private void addListener() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        SinkListenerServiceLocator notificationService =
            new SinkListenerServiceLocator();

        try {
            SinkListenerPortType sinkListener =
                notificationService.getSinkListenerPort(this.defaultEndpoint);
            GSIUtils.setDefaultGSIProperties(
                (Stub) sinkListener, this.defaultEndpoint
            );
            sinkListener.addListener(
                this.notificationTopicTF.getText(),
                new HandleType(this.notificationSourceTF.getText())
            );
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to add listener: " + e.getMessage(),
                "SinkListener: addListener error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }

        JOptionPane.showMessageDialog(
            this,
            "Successfully added Listener to: " +
            this.notificationSourceTF.getText() + " - " +
            this.notificationTopicTF.getText(), "SinkListener: addListener",
            JOptionPane.INFORMATION_MESSAGE
        );
    }

    private void removeListener() {
        if ((this.context == null) || (this.defaultEndpoint == null)) {
            // TODO: defaultEndpoint error
            return;
        }

        SinkListenerServiceLocator notificationService =
            new SinkListenerServiceLocator();

        try {
            SinkListenerPortType sinkListener =
                notificationService.getSinkListenerPort(this.defaultEndpoint);
            GSIUtils.setDefaultGSIProperties(
                (Stub) sinkListener, this.defaultEndpoint
            );
            sinkListener.removeListener(
                notificationTopicTF.getText(),
                new HandleType(notificationSourceTF.getText())
            );
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                this, "Failed to remove listener: " + e.getMessage(),
                "SinkListener: removeListener error", JOptionPane.ERROR_MESSAGE
            );
            e.printStackTrace();

            return;
        }

        JOptionPane.showMessageDialog(
            this,
            "Successfully removed Listener from: " +
            this.notificationSourceTF.getText() + " - " +
            this.notificationTopicTF.getText(), "SinkListener: removeListener",
            JOptionPane.INFORMATION_MESSAGE
        );
    }

    public void init() {
    }

    public void dispose() {
    }
}
