/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.TreeCellRenderer;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;


public class XMLTreeCellRenderer implements TreeCellRenderer {
    public XMLTreeCellRenderer() {
    }

    public Component getTreeCellRendererComponent(
        JTree tree,
        Object value,
        boolean selected,
        boolean expanded,
        boolean leaf,
        int row,
        boolean hasFocus
    ) {
        Node node = (Node) value;
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 5));

        // get element
        String nodeName = getName(node);

        if ((nodeName != null) && (!nodeName.equals(""))) {
            JLabel nodeLabel = new JLabel(getName(node));
            nodeLabel.setForeground(Color.red);
            panel.add(nodeLabel);
        }

        // get attributes
        NamedNodeMap map = node.getAttributes();

        for (int i = 0; (map != null) && (i < map.getLength()); i++) {
            Node attributeNode = map.item(i);
            JLabel attributeLabel = new JLabel(" " + getName(attributeNode));
            attributeLabel.setForeground(Color.blue);
            panel.add(attributeLabel);

            JLabel equalLabel = new JLabel("=");
            equalLabel.setForeground(Color.red);
            panel.add(equalLabel);

            JLabel attributeValueLabel =
                new JLabel("\"" + attributeNode.getNodeValue() + "\"");
            attributeValueLabel.setForeground(Color.black);
            panel.add(attributeValueLabel);
        }

        // get text
        if (node.getNodeType() == Node.TEXT_NODE) {
            String nodeValue = node.getNodeValue();

            if ((nodeValue != null) && !nodeValue.equals("")) {
                JLabel nodeValueLabel = new JLabel("  " + nodeValue);
                nodeValueLabel.setForeground(Color.green);
                panel.add(nodeValueLabel);
            }
        }

        panel.setBackground(Color.white);

        return panel;
    }

    private String getName(Node node) {
        String localName = node.getLocalName();

        if (localName == null) {
            return null;
        }

        String ns = node.getPrefix();
        String tag = "";

        if ((ns != null) && (!ns.equals(""))) {
            tag = ns + ":";
        }

        return tag + localName;
    }
}
;
