/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.globus.ogsa.utils.XmlFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import java.util.Vector;


public class XMLTreeModel implements TreeModel {
    private Node rootNode;
    Vector treeModelListeners = new Vector();

    public XMLTreeModel(Node rootNode) {
        if (!(rootNode instanceof Document)) {
            try {
                this.rootNode = XmlFactory.newDocument();
            } catch (Exception e) {
                e.printStackTrace();
            }

            Node importNode =
                ((Document) this.rootNode).importNode(rootNode, true);
            this.rootNode.appendChild(importNode);
        } else {
            this.rootNode = rootNode;
        }

        this.rootNode.normalize();
        trimNode(this.rootNode);
    }

    protected void fireTreeStructureChanged(Node oldRoot) {
        int len = treeModelListeners.size();
        TreeModelEvent e = new TreeModelEvent(this, new Object[] { oldRoot });

        for (int i = 0; i < len; i++) {
            ((TreeModelListener) treeModelListeners.elementAt(i)).treeStructureChanged(
                e
            );
        }
    }

    public void addTreeModelListener(TreeModelListener listener) {
        treeModelListeners.addElement(listener);
    }

    public Object getChild(
        Object parent,
        int index
    ) {
        NodeList nodes = ((Node) parent).getChildNodes();

        return nodes.item(index);
    }

    public int getChildCount(Object parent) {
        NodeList nodes = ((Node) parent).getChildNodes();

        return nodes.getLength();
    }

    public int getIndexOfChild(
        Object parent,
        Object child
    ) {
        NodeList nodes = ((Node) parent).getChildNodes();

        for (int i = 0; i < nodes.getLength(); i++) {
            if (isEqual(nodes.item(i), (Node) child)) {
                return i;
            }
        }

        return -1;
    }

    private boolean isEqual(
        Node node1,
        Node node2
    ) {
        String name1 = node1.getLocalName();
        String NS1 = node1.getNamespaceURI();
        String name2 = node2.getLocalName();
        String NS2 = node2.getNamespaceURI();

        return (name1.equals(name2) &&
        (((NS1 == null) && (NS2 == null)) || NS1.equals(NS2)));
    }

    public Object getRoot() {
        return rootNode;
    }

    public boolean isLeaf(Object node) {
        NodeList nodes = ((Node) node).getChildNodes();

        return (nodes.getLength() == 0);
    }

    public void removeTreeModelListener(TreeModelListener listener) {
        treeModelListeners.removeElement(listener);
    }

    public void valueForPathChanged(
        TreePath path,
        Object newValue
    ) {
    }

    private void trimNode(Node node) {
        NodeList children = node.getChildNodes();

        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i);

            if (child.getChildNodes().getLength() != 0) {
                trimNode(child);
            }

            if (child.getNodeType() == Node.TEXT_NODE) {
                if (emptyNode(child)) {
                    node.removeChild(child);
                    i--;
                }
            }
        }
    }

    private boolean emptyNode(Node node) {
        String raw = node.getNodeValue();

        if (raw == null) {
            return true;
        }

        raw.trim();

        if (raw.equals("")) {
            return true;
        }

        char[] chars = raw.toCharArray();

        for (int i = 0; i < chars.length; i++) {
            if (
                Character.isISOControl(chars[i]) ||
                    Character.isWhitespace(chars[i])
            ) {
                continue;
            }

            return false;
        }

        return true;
    }
}
